@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -35,7 +35,7 @@ public:
    bool bdb_open_database(JCR *jcr);
    void bdb_close_database(JCR *jcr);
    void bdb_thread_cleanup(void);
-   void bdb_escape_string(JCR *jcr, char *snew, char *old, int len);
+   void bdb_escape_string(JCR *jcr, char *snew, const char *old, int len);
    char *bdb_escape_object(JCR *jcr, char *old, int len);
    void bdb_unescape_object(JCR *jcr, char *from, int32_t expected_len,
                            POOLMEM **dest, int32_t *len);
@@ -59,6 +59,7 @@ public:
    bool sql_batch_start(JCR *jcr);
    bool sql_batch_end(JCR *jcr, const char *error);
    bool sql_batch_insert(JCR *jcr, ATTR_DBR *ar);
+   const char *search_op(JCR *jcr, const char *table_col, char *value, POOLMEM **esc, POOLMEM **dest);
 };
 
 #endif /* __BDB_POSTGRESQL_H_ */
