@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -60,6 +60,7 @@ typedef enum {
    BVFS_Md5     = 6,
    BVFS_VolName = 7,
    BVFS_VolInchanger = 8,
+   BVFS_VolType  = 9,
 
    /* Only if Delta record */
    BVFS_DeltaSeq = 6,
@@ -241,7 +242,7 @@ public:
    void clear_cache();
 
    /* Compute restore list */
-   bool compute_restore_list(char *fileid, char *dirid, char *output_table);
+   bool compute_restore_list(char *fileid, char *dirid, char *fileindex, char *output_table);
 
    /* Drop previous restore list */
    bool drop_restore_list(char *output_table);
