@@ -1,7 +1,7 @@
 /* 
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -276,13 +276,14 @@ bool BDB_MYSQL::bdb_open_database(JCR *jcr)
       bmicrosleep(5,0); 
    } 
 
-#if MYSQL_VERSION_ID <= 50117
+#if MYSQL_VERSION_ID  <   80034 // Deprecated in 8.0.34
+# if MYSQL_VERSION_ID <=  50117
    mysql_options(&mdb->m_instance, MYSQL_OPT_RECONNECT, (char*)&reconnect); /* so connection does not timeout */
-#else
+# else
    mysql_options(&mdb->m_instance, MYSQL_OPT_RECONNECT, &reconnect); /* so connection does not timeout */
-#endif
-
+# endif
    Dmsg0(50, "mysql_real_connect done\n");
+#endif
    Dmsg3(50, "db_user=%s db_name=%s db_password=%s\n", mdb->m_db_user, mdb->m_db_name, 
         (mdb->m_db_password == NULL) ? "(NULL)" : mdb->m_db_password); 
  
@@ -441,7 +442,7 @@ void BDB_MYSQL::bdb_thread_cleanup(void)
  *   string must be long enough (max 2*old+1) to hold 
  *   the escaped output. 
  */ 
-void BDB_MYSQL::bdb_escape_string(JCR *jcr, char *snew, char *old, int len) 
+void BDB_MYSQL::bdb_escape_string(JCR *jcr, char *snew, const char *old, int len) 
 { 
    BDB_MYSQL *mdb = this; 
    mysql_real_escape_string(mdb->m_db_handle, snew, old, len); 
@@ -853,5 +854,13 @@ bool BDB_MYSQL::sql_batch_insert(JCR *jcr, ATTR_DBR *ar)
    return true; 
 } 
  
+const char *BDB_MYSQL::search_op(JCR *jcr, const char *table_col, char *value, POOLMEM **esc, POOLMEM **dest)
+{
+   int len = strlen(value);
+   *esc = check_pool_memory_size(*esc, len*2+1);
+   bdb_escape_string(jcr, *esc, value, len);
+   Mmsg(dest, " MATCH(%s) AGAINST ('%s') ", table_col, *esc);
+   return *dest;
+}
  
 #endif /* HAVE_MYSQL */ 
