@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -92,8 +92,8 @@ int db_int_handler(void *ctx, int num_fields, char **row);
 void bdb_debug_print(JCR *jcr, FILE *fp);
 void db_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
 
-#define db_open_batch_connexion(jcr, mdb) \
-           mdb->bdb_open_batch_connexion(jcr)
+#define db_open_batch_connection(jcr, mdb) \
+           mdb->bdb_open_batch_connection(jcr)
 #define db_strerror(mdb) \
            mdb->bdb_strerror()
 #define db_debug_print(jcr, fp) \
@@ -164,7 +164,8 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            bdb_disable_batch_insert(disable)
 #define db_create_snapshot_record(jcr, mdb, sr) \
            mdb->bdb_create_snapshot_record(jcr, sr)
-
+#define db_create_fileevent_record(jcr, mdb, ev) \
+           mdb->bdb_create_fileevent_record(jcr, ev)
 
 /* sql_delete.c */
 #define db_delete_pool_record(jcr, mdb, pool_dbr) \
@@ -216,8 +217,8 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            mdb->bdb_get_jobmedia_record(jcr, jmr)
 #define db_get_job_record(jcr, mdb, jr) \
            mdb->bdb_get_job_record(jcr, jr)
-#define db_get_job_volume_names(jcr, mdb, JobId, VolumeNames) \
-           mdb->bdb_get_job_volume_names(jcr, JobId, VolumeNames)
+#define db_get_job_volume_names(jcr, mdb, JobId, VolumeNames, LastVolumeName, maxlen) \
+           mdb->bdb_get_job_volume_names(jcr, JobId, VolumeNames, LastVolumeName, maxlen)
 #define db_get_file_attributes_record(jcr, mdb, fname, jr, fdbr) \
            mdb->bdb_get_file_attributes_record(jcr, fname, jr, fdbr)
 #define db_get_fileset_record(jcr, mdb, fsr) \
@@ -240,7 +241,9 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            mdb->bdb_get_counter_record(jcr, cr)
 #define db_get_query_dbids(jcr, mdb, query, ids) \
            mdb->bdb_get_query_dbids(jcr, query, ids)
-#define db_get_file_list(jcr, mdb, jobids, opts, result_handler, ctx) \
+#define db_get_prior_job(jcr, mdb, jobids, jr) \
+           mdb->bdb_get_prior_job(jcr, jobids, jr)
+#define db_get_file_list(jcr, mdb, jobids, opts, result_handler, ctx)   \
            mdb->bdb_get_file_list(jcr, jobids, opts, result_handler, ctx)
 #define db_get_base_jobid(jcr, mdb, jr, jobid) \
            mdb->bdb_get_base_jobid(jcr, jr, jobid)
@@ -266,10 +269,19 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            mdb->bdb_get_snapshot_record(jcr, sr)
 #define db_get_job_statistics(jcr, mdb, jr)      \
            mdb->bdb_get_job_statistics(jcr, jr)
+
 /* sql_list.c */
 #define db_list_jobs_for_file(jcr, mdb, cli, fname, result_handler, ctx, type) \
            mdb->bdb_list_jobs_for_file(jcr, cli, fname, result_handler, ctx, type)
-#define db_list_pool_records(jcr, mdb, pr, sendit, ctx, type) \
+#define db_search_tag_records(jcr, mdb, tag, result_handler, ctx)      \
+           mdb->bdb_search_tag_records(jcr, tag, result_handler, ctx)
+#define db_search_media_records(jcr, mdb, rec, result_handler, ctx)      \
+           mdb->bdb_search_media_records(jcr, rec, result_handler, ctx)
+#define db_search_job_records(jcr, mdb, rec, result_handler, ctx)      \
+           mdb->bdb_search_job_records(jcr, rec, result_handler, ctx)
+#define db_search_client_records(jcr, mdb, rec, result_handler, ctx)      \
+           mdb->bdb_search_client_records(jcr, rec, result_handler, ctx)
+#define db_list_pool_records(jcr, mdb, pr, sendit, ctx, type)           \
            mdb->bdb_list_pool_records(jcr, pr, sendit, ctx, type)
 #define db_list_job_records(jcr, mdb, jr, sendit, ctx, type) \
            mdb->bdb_list_job_records(jcr, jr, sendit, ctx, type)
@@ -277,14 +289,18 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            mdb->bdb_list_job_totals(jcr, jr, sendit, ctx)
 #define db_list_files_for_job(jcr, mdb, jobid, deleted, sendit, ctx)     \
            mdb->bdb_list_files_for_job(jcr, jobid, deleted, sendit, ctx)
+#define db_list_fileevents_for_job(jcr, mdb, jobid, etype, sendit, ctx, type) \
+           mdb->bdb_list_fileevents_for_job(jcr, jobid, etype, sendit, ctx, type)
 #define db_list_media_records(jcr, mdb, mdbr, sendit, ctx, type) \
            mdb->bdb_list_media_records(jcr, mdbr, sendit, ctx, type)
 #define db_list_jobmedia_records(jcr, mdb, JobId, volume, sendit, ctx, type) \
            mdb->bdb_list_jobmedia_records(jcr, JobId, volume, sendit, ctx, type)
 #define db_list_filemedia_records(jcr, mdb, JobId, FI, sendit, ctx, type) \
            mdb->bdb_list_filemedia_records(jcr, JobId, FI, sendit, ctx, type)
-#define db_list_joblog_records(jcr, mdb, JobId, sendit, ctx, type)      \
-           mdb->bdb_list_joblog_records(jcr, JobId, sendit, ctx, type)
+#define db_list_fileevents_records(jcr, mdb, event, sendit, ctx, type) \
+           mdb->bdb_list_fileevents_records(jcr, event, sendit, ctx, type)
+#define db_list_joblog_records(jcr, mdb, JobId, pattern, sendit, ctx, type) \
+   mdb->bdb_list_joblog_records(jcr, JobId, pattern, sendit, ctx, type)
 #define db_list_sql_query(jcr, mdb, title, query, sendit, ctx, verbose, type) \
            mdb->bdb_list_sql_query(jcr, title, query, sendit, ctx, verbose, type)
 #define db_list_client_records(jcr, mdb, sendit, ctx, type) \
@@ -309,6 +325,8 @@ void bdb_free_restoreobject_record(JCR *jcr, ROBJECT_DBR *rr);
            mdb->bdb_list_files(jcr, filedbr, sendit, ua);
 #define db_list_tag_records(jcr, mdb, tagdbr, sendit, ua, llist)     \
            mdb->bdb_list_tag_records(jcr, tagdbr, sendit, ua, llist);
+#define db_list_metadata_records(jcr, mdb, dbr, sendit, ua, llist)     \
+           mdb->bdb_list_metadata_records(jcr, dbr, sendit, ua, llist);
 
 /* sql_update.c */
 #define db_update_job_start_record(jcr, mdb, jr) \
