@@ -1,7 +1,7 @@
 /* 
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -1258,12 +1258,12 @@ json_list:
    json_list_end(send, ctx, 0, "");
    return mdb->sql_num_rows();
 } 
- 
+
 /* 
- * Open a new connexion to mdb catalog. This function is used 
+ * Open a new connection to mdb catalog. This function is used 
  * by batch and accurate mode. 
  */ 
-bool BDB::bdb_open_batch_connexion(JCR *jcr) 
+bool BDB::bdb_open_batch_connection(JCR *jcr) 
 { 
    bool multi_db; 
  
@@ -1413,6 +1413,15 @@ void TAG_DBR::gen_sql(JCR *jcr, BDB *db,
    *aclbits_extra_ = aclbits_extra;
 }
 
+const char *BDB::search_op(JCR *jcr, const char *table_col, char *value, POOLMEM **esc, POOLMEM **dest)
+{
+   int len = strlen(value);
+   *esc = check_pool_memory_size(*esc, len*2+1);
+   bdb_escape_string(jcr, *esc, (char*)value, len);
+   Mmsg(dest, " %s ILIKE '%%%s%%'", table_col, value);
+   return *dest;
+}
+
 #ifdef COMMUNITY
 bool BDB::bdb_check_settings(JCR *jcr, int64_t *starttime, int val, int64_t val2) 
 {
