@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -47,19 +47,43 @@ const char *cleanup_created_job =
 const char *cleanup_running_job = 
    "UPDATE Job SET JobStatus='f', EndTime=StartTime WHERE JobStatus = 'R'";
 
+const char *sql_clear_malware_table_def = "TRUNCATE Malware%s";
+const char *sql_clear_malware_table[] = {
+   "DELETE * FROM Malware%s",   // Truncate on Mysql doesn't work with transaction
+   sql_clear_malware_table_def,
+   sql_clear_malware_table_def
+};
+
+const char *sync_jobhisto_def = "UPDATE JobHisto SET PurgedFiles=Job.PurgedFiles, HasCache=Job.HasCache,Reviewed=Job.Reviewed "
+    "FROM Job WHERE Job.JobId=JobHisto.JobId and Job.Job = JobHisto.Job "
+      "AND (Job.HasCache <> JobHisto.HasCache OR Job.PurgedFiles <> JobHisto.PurgedFiles OR Job.Reviewed <> JobHisto.Reviewed)";
+const char *sync_jobhisto[] =
+{
+   // MySQL
+   "UPDATE JobHisto JOIN Job USING (JobId) SET JobHisto.PurgedFiles=Job.PurgedFiles, JobHisto.HasCache=Job.HasCache, JobHisto.Reviewed=Job.Reviewed "
+   "WHERE Job.JobId=JobHisto.JobId and Job.Job = JobHisto.Job "
+   "AND (Job.HasCache <> JobHisto.HasCache OR Job.PurgedFiles <> JobHisto.PurgedFiles OR Job.Reviewed <> JobHisto.Reviewed)",
+   // PostgreSQL
+   sync_jobhisto_def,
+   // SQLite
+   sync_jobhisto_def
+};
+
 /* For sql_update.c db_update_stats */
 const char *fill_jobhisto =
         "INSERT INTO JobHisto (JobId, Job, Name, Type, Level,"
            "ClientId, JobStatus,"
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate,"
+           "SchedTime, StartTime, EndTime, RealEndTime, RealStartTime, JobTDate,"
            "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes,"
            "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, PriorJob, "
-           "PurgedFiles, HasBase, Reviewed, Comment)"
+           "PurgedFiles, HasBase, HasCache, Reviewed, Comment, FileTable, "
+           "isVirtualFull,CompressRatio,Rate,LastReadStorageId,WriteStorageId,LastReadDevice,WriteDevice,StatusInfo,Encrypted)"
         "SELECT JobId, Job, Name, Type, Level, ClientId, JobStatus,"
-           "SchedTime, StartTime, EndTime, RealEndTime, JobTDate,"
+           "SchedTime, StartTime, EndTime, RealEndTime, RealStartTime, JobTDate,"
            "VolSessionId, VolSessionTime, JobFiles, JobBytes, ReadBytes,"
            "JobErrors, JobMissingFiles, PoolId, FileSetId, PriorJobId, PriorJob, "
-           "PurgedFiles, HasBase, Reviewed, Comment "
+           "PurgedFiles, HasBase, HasCache, Reviewed, Comment, FileTable, "
+           "isVirtualFull,CompressRatio,Rate,LastReadStorageId,WriteStorageId,LastReadDevice,WriteDevice,StatusInfo,Encrypted "
           "FROM Job "
          "WHERE JobStatus IN ('T','W','f','A','E')"
            "AND NOT EXISTS "
@@ -739,13 +763,13 @@ static const char *sql_bvfs_list_files_default =
 "SELECT 'F', T.PathId, T.Filename, "
         "File.JobId, File.LStat, File.FileId "
 "FROM Job, File, ( "
-    "SELECT MAX(JobTDate) AS JobTDate, PathId, Filename "
+    "SELECT MAX(JobTDate) AS JobTDate, MAX(DeltaSeq) AS DeltaSeq, PathId, Filename "
       "FROM ( "
-        "SELECT JobTDate, PathId, Filename "
+        "SELECT JobTDate, PathId, Filename, DeltaSeq "
           "FROM File JOIN Job USING (JobId) "
          "WHERE File.JobId IN (%s) AND PathId = %s "
           "UNION ALL "
-        "SELECT JobTDate, PathId, Filename "
+        "SELECT JobTDate, PathId, Filename, DeltaSeq "
           "FROM BaseFiles "
                "JOIN File USING (FileId) "
                "JOIN Job  ON    (BaseJobId = Job.JobId) "
@@ -753,6 +777,7 @@ static const char *sql_bvfs_list_files_default =
        ") AS tmp GROUP BY PathId, Filename "
     ") AS T "
 "WHERE T.JobTDate = Job.JobTDate "
+  "AND T.DeltaSeq = File.DeltaSeq "
   "AND Job.JobId = File.JobId "
   "AND T.PathId = File.PathId "
   "AND T.Filename = File.Filename "
@@ -774,23 +799,20 @@ const char *sql_bvfs_list_files[] = {
    "SELECT DISTINCT ON (Filename) 'F' as Type, PathId, FileId, "
     "T.Filename, JobId, LStat, FileIndex "
      "FROM "
-         "(SELECT FileId, JobId, PathId, Filename, FileIndex, LStat, MD5 "
+         "(SELECT FileId, JobId, PathId, Filename, FileIndex, LStat, MD5, DeltaSeq "
             "FROM File WHERE JobId IN (%s) AND PathId = %s "
            "UNION ALL "
           "SELECT File.FileId, File.JobId, PathId, Filename, "
-                 "File.FileIndex, LStat, MD5 "
+                 "File.FileIndex, LStat, MD5, DeltaSeq "
             "FROM BaseFiles JOIN File USING (FileId) "
            "WHERE BaseFiles.JobId IN (%s) AND File.PathId = %s "
           ") AS T JOIN Job USING (JobId) "
           " WHERE T.Filename != '' "
           " %s "               /* AND Filename LIKE '' */
-     "ORDER BY Filename, StartTime DESC "
+     "ORDER BY Filename, DeltaSeq DESC, StartTime DESC "
    ") AS A WHERE A.FileIndex > 0 "
    "LIMIT %lld OFFSET %lld ",
 
-   /* SQLite */
-   sql_bvfs_list_files_default,
-
    /* SQLite */
    sql_bvfs_list_files_default
 };
@@ -910,7 +932,17 @@ const char *match_query[] =
    /* SQLite */
    "LIKE"                       /* MATCH doesn't seems to work anymore... */
 }; 
- 
+
+const char *sql_like[] =
+{
+   /* MySQL */
+   "LIKE",
+   /* PostgreSQL */
+   "ILIKE",
+   /* SQLite */
+   "LIKE"
+};
+
 static const char *insert_counter_values_default =
    "INSERT INTO Counters (Counter, MinValue, "
    "MaxValue, CurrentValue, WrapCounter) "
@@ -961,6 +993,15 @@ const char *update_counter_values[] =
    update_counter_values_default 
 }; 
 
+const char *sql_now[] = {
+   /* MySQL */
+   " NOW() ",
+   /* PostgreSQL */
+   " NOW() ",
+   /* SQLite */
+   " strftime('%s', datetime('now', 'localtime')) "
+};
+
 const char *prune_cache[] = {
    /* MySQL */
    " (Media.LastWritten + Media.CacheRetention) < NOW() ",
@@ -1072,3 +1113,43 @@ const char *regexp_value[] = {
    /* SQLite */
    regexp_value_default
 };
+
+/* Here we choose the last Object for a list of Job, we check if the object points to a place
+ * that still exists. We take the last version of them
+ */
+
+/* In the MySQL version, we cannot use DISTINCT ON(), so, when we have our MAX(JobTDate) (last record), we
+ * need to find back the other values by joining back to Object. Of course, if we have multiple objects
+ * with the same kind of attributes for the same job, it can cause a problem.
+ */
+const char *copy_object_default =
+   "INSERT INTO Object (JobId, Path, Filename, PluginName, ObjectCategory, ObjectType, ObjectName, ObjectSource, ObjectUUID, "
+       " ObjectSize, ObjectStatus, ObjectCount) "
+   // If we rename a VM for example, the Path and the Filename will stay, but not the Name or maybe the PluginName
+   "SELECT %lu, A.Path, A.Filename, Object.PluginName, A.ObjectCategory, A.ObjectType, Object.ObjectName, Object.ObjectSource, Object.ObjectUUID, Object.ObjectSize, Object.ObjectStatus, Object.ObjectCount "
+     "FROM (SELECT O.Path, O.Filename, O.ObjectCategory, O.ObjectType, MAX(J.JobTDate) AS JobTDate "
+             "FROM Object AS O JOIN Path USING (Path) JOIN Job AS J USING (JobId) WHERE J.JobId IN (%s) AND O.Path = Path.Path AND "
+             "EXISTS (SELECT 1 FROM File WHERE O.JobId = File.JobId AND Path.PathId = File.PathId AND (O.Filename = File.Filename OR O.Filename = '')) "
+            "GROUP BY O.Path, O.Filename, O.ObjectCategory, O.ObjectType) AS A "
+     "JOIN Job USING (JobTDate) JOIN Object ON (Job.JobId = Object.JobId AND Object.Path = A.Path AND Object.Filename = A.Filename AND A.ObjectCategory = Object.ObjectCategory AND A.ObjectType = Object.ObjectType) WHERE Job.JobId IN (%s)";
+
+const char *copy_object[] = {
+   /* MySQL */
+   copy_object_default,
+
+   /* PostgreSQL */
+   "INSERT INTO Object (JobId, Path, Filename, PluginName, ObjectCategory, ObjectType, ObjectName, ObjectSource, ObjectUUID, "
+       " ObjectSize, ObjectStatus, ObjectCount) "
+   // If we rename a VM for example, the Path and the Filename will stay, but not the Name or maybe the PluginName
+   "SELECT DISTINCT ON (Object.Path, Object.Filename, ObjectCategory, ObjectType) "
+     " %lu, Object.Path, Object.Filename, PluginName, ObjectCategory, ObjectType, ObjectName, ObjectSource, ObjectUUID, "
+     " ObjectSize, ObjectStatus, ObjectCount "
+   "FROM Object JOIN Path USING (Path) JOIN Job USING (JobId) "
+  "WHERE Job.JobId IN (%s) AND Object.Path = Path.Path "
+   // In some plugins, the filename is empty but the associated record in the DB doesn't exist
+    "AND EXISTS (SELECT 1 FROM File WHERE Object.JobId = File.JobId AND Path.PathId = File.PathId AND (Object.Filename = File.Filename OR Object.Filename = '')) "
+   "ORDER BY Object.Path, Object.Filename, ObjectCategory, ObjectType, JobTDate DESC",
+
+   /* SQLite */
+   copy_object_default
+};
