@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -78,11 +78,11 @@ bool BDB::bdb_create_job_record(JCR *jcr, JOB_DBR *jr)
    /* Must create it */
    Mmsg(cmd,
 "INSERT INTO Job (Job,Name,Type,Level,JobStatus,StartTime,SchedTime,JobTDate,"
-                 "ClientId,Comment) "
-"VALUES ('%s','%s','%c','%c','%c','%s','%s',%s,%s,'%s')",
+                 "ClientId,Comment,isVirtualFull) "
+"VALUES ('%s','%s','%c','%c','%c','%s','%s',%s,%s,'%s',%d)",
            esc_job, esc_name, (char)(jr->JobType), (char)(jr->JobLevel),
           (char)(jr->JobStatus), dt2, dt, edit_uint64(JobTDate, ed1),
-           edit_int64(jr->ClientId, ed2), buf.c_str());
+          edit_int64(jr->ClientId, ed2), buf.c_str(), jr->isVirtualFull);
 
    if ((jr->JobId = sql_insert_autokey_record(cmd, NT_("Job"))) == 0) {
       Mmsg2(&errmsg, _("Create DB Job record %s failed. ERR=%s\n"),
@@ -442,9 +442,10 @@ int BDB::bdb_create_media_record(JCR *jcr, MEDIA_DBR *mr)
 "VolStatus,Slot,VolBytes,InChanger,VolReadTime,VolWriteTime,VolType,"
 "VolParts,VolCloudParts,LastPartBytes,"
 "EndFile,EndBlock,LabelType,StorageId,DeviceId,LocationId,"
-"ScratchPoolId,RecyclePoolId,Enabled,ActionOnPurge,CacheRetention)"
-"VALUES ('%s','%s',0,%u,%s,%s,%d,%s,%s,%u,%u,'%s',%d,%s,%d,%s,%s,%d,"
-        "%d,%d,'%s',%d,%d,%d,%s,%s,%s,%s,%s,%d,%d,%s)",
+"ScratchPoolId,RecyclePoolId,Enabled,ActionOnPurge,CacheRetention,UseProtect,"
+"Protected,VolEncrypted)"
+"VALUES ('%s','%s',0,%lu,%s,%s,%ld,%s,%s,%lu,%lu,'%s',%ld,%s,%ld,%s,%s,%ld,"
+        "%ld,%ld,'%s',%ld,%ld,%ld,%s,%s,%s,%s,%s,%ld,%ld,%s,%ld,%ld,%ld)",
           esc_name,
           esc_mtype, mr->PoolId,
           edit_uint64(mr->MaxVolBytes,ed1),
@@ -472,9 +473,12 @@ int BDB::bdb_create_media_record(JCR *jcr, MEDIA_DBR *mr)
           edit_int64(mr->LocationId, ed11),
           edit_int64(mr->ScratchPoolId, ed12),
           edit_int64(mr->RecyclePoolId, ed13),
-          mr->Enabled, 
+          mr->Enabled,
           mr->ActionOnPurge,
-          edit_uint64(mr->CacheRetention, ed14)
+          edit_uint64(mr->CacheRetention, ed14),
+          mr->UseProtect,
+          mr->Protected,
+          mr->VolEncrypted
           );
 
 
@@ -729,13 +733,15 @@ bool BDB::bdb_create_fileset_record(JCR *jcr, FILESET_DBR *fsr)
    struct tm tm;
    char esc_fs[MAX_ESCAPE_NAME_LENGTH];
    char esc_md5[MAX_ESCAPE_NAME_LENGTH];
+   char esc_content[MAX_ESCAPE_PLUGIN_LENGTH];
 
    /* TODO: Escape FileSet and MD5 */
    bdb_lock();
    fsr->created = false;
    bdb_escape_string(jcr, esc_fs, fsr->FileSet, strlen(fsr->FileSet));
    bdb_escape_string(jcr, esc_md5, fsr->MD5, strlen(fsr->MD5));
-   Mmsg(cmd, "SELECT FileSetId,CreateTime FROM FileSet WHERE "
+   bdb_escape_string(jcr, esc_content, fsr->Content, strlen(fsr->Content));
+   Mmsg(cmd, "SELECT FileSetId,CreateTime,Content FROM FileSet WHERE "
                   "FileSet='%s' AND MD5='%s'", esc_fs, esc_md5);
 
    fsr->FileSetId = 0;
@@ -758,6 +764,17 @@ bool BDB::bdb_create_fileset_record(JCR *jcr, FILESET_DBR *fsr)
          } else {
             bstrncpy(fsr->cCreateTime, row[1], sizeof(fsr->cCreateTime));
          }
+         if ((row[2] == NULL || strcmp(row[2], "") == 0) && esc_content[0]) {
+            Mmsg(cmd, "UPDATE FileSet SET Content='%s' WHERE FileSetId=%ld",
+                 esc_content, fsr->FileSetId);
+            if (!UpdateDB(jcr, cmd, false /* should match*/)) {
+               /* If we cannot update, this is not the end of the world, but we
+                * can display a message
+                */
+               Dmsg2(50, "Unable to update FileSet content field for %ld ERR=%s\n",
+                     fsr->FileSetId, sql_strerror());
+            }
+         }
          sql_free_result();
          bdb_unlock();
          return true;
@@ -772,8 +789,8 @@ bool BDB::bdb_create_fileset_record(JCR *jcr, FILESET_DBR *fsr)
    strftime(fsr->cCreateTime, sizeof(fsr->cCreateTime), "%Y-%m-%d %H:%M:%S", &tm);
 
    /* Must create it */
-      Mmsg(cmd, "INSERT INTO FileSet (FileSet,MD5,CreateTime) "
-"VALUES ('%s','%s','%s')", esc_fs, esc_md5, fsr->cCreateTime);
+      Mmsg(cmd, "INSERT INTO FileSet (FileSet,MD5,CreateTime,Content) "
+           "VALUES ('%s','%s','%s','%s')", esc_fs, esc_md5, fsr->cCreateTime, esc_content);
 
    if ((fsr->FileSetId = sql_insert_autokey_record(cmd, NT_("FileSet"))) == 0) {
       Mmsg2(&errmsg, _("Create DB FileSet record %s failed. ERR=%s\n"),
@@ -930,7 +947,7 @@ bool BDB::bdb_create_batch_file_attributes_record(JCR *jcr, ATTR_DBR *ar)
 
    /* Open the dedicated connexion */
    if (!jcr->batch_started) {
-      if (!bdb_open_batch_connexion(jcr)) {
+      if (!bdb_open_batch_connection(jcr)) {
          return false;     /* error already printed */
       }
       if (!jcr->db_batch->sql_batch_start(jcr)) {
@@ -1460,4 +1477,21 @@ bool BDB::bdb_create_tag_record(JCR *jcr, TAG_DBR *tag)
    return ret;
 }
 
+bool BDB::bdb_create_fileevent_record(JCR *jcr, FILEEVENT_DBR *event)
+{
+   char esc1[MAX_ESCAPE_NAME_LENGTH];
+   char esc2[MAX_ESCAPE_NAME_LENGTH];
+   bool ret=false;
+
+   bdb_lock();
+   bdb_escape_string(jcr, esc1, event->Description, strlen(event->Description));
+   bdb_escape_string(jcr, esc2, event->Source, strlen(event->Source));
+   Mmsg(cmd, "INSERT INTO FileEvents (SourceJobId, JobId, FileIndex, Type, Description, Severity, Source) "
+        " VALUES (%lu, %s, %ld, '%c', '%s', %d, '%s')",
+        event->SourceJobId, event->JobId, event->FileIndex, event->Type, esc1, event->Severity, esc2);
+   ret = bdb_sql_query(cmd, NULL, (void *)NULL);
+   bdb_unlock();
+   return ret;
+}
+
 #endif /* HAVE_SQLITE3 || HAVE_MYSQL || HAVE_POSTGRESQL */
