@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -20,6 +20,7 @@
  * Includes specific to the Director
  *
  *     Kern Sibbald, December MM
+ *
  */
 
 #include "lib/ini.h"
@@ -60,14 +61,6 @@ struct del_ctx {
    int tot_ids;                       /* total to process */
 };
 
-/* Flags for find_next_volume_for_append() */
-enum {
-  fnv_create_vol    = true,
-  fnv_no_create_vol = false,
-  fnv_prune         = true,
-  fnv_no_prune      = false
-};
-
 typedef struct {
    char    *plugin_name;
    POOLMEM *content;
