@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -126,6 +126,16 @@ static int job_item(JCR *jcr, int code,
    case 12:                           /* JobName */
       str = jcr->Job;
       break;
+   case 13:
+      str = jcr->previous_jr.Name;     /* Previous Job name */
+      break;
+   case 14:
+      bsnprintf(buf, sizeof(buf), "%d", jcr->previous_jr.JobId); /* Previous JobId */
+      str = buf;
+      break;
+   case 15:			// Return the timestamp associated with the job name
+      str = jcr->Job + strlen(jcr->Job) - strlen("2023-10-12_20.27.20_59");
+      break;
    }
    *val_ptr = bstrdup(str);
    *val_len = strlen(str);
@@ -160,7 +170,9 @@ static struct s_built_in_vars built_in_vars[] = {
    { NT_("Catalog"),   10, job_item},
    { NT_("MediaType"), 11, job_item},
    { NT_("JobName"),   12, job_item},
-
+   { NT_("PriorJobName"), 13, job_item},
+   { NT_("PriorJobId"), 14, job_item},
+   { NT_("JobTimestamp"), 15, job_item},
    { NULL, 0, NULL}
 };
 
