@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -57,7 +57,6 @@ static char Device_update[]   = "DevUpd JobId=%127s "
    "DevWriteBytes=%d\n";
 #endif
 
-
 static char OK_msg[] = "1000 OK\n";
 
 
@@ -276,9 +275,9 @@ int bget_dirmsg(JCR *jcr, BSOCK *bs, BSOCK_CLIENT_TYPE role)
          catalog_request(jcr, bs);
          continue;
       }
-      /* Only the Snapshot commands are authorized for the FD */
+      /* Only the Snapshot and FileEvent commands are authorized for the FD */
       if (role==BSOCK_TYPE_FD && bs->msg[0] == 'C') {
-         snapshot_catreq(jcr, bs);
+         fd_catreq(jcr, bs);
          continue;
       }
       if (role==BSOCK_TYPE_SD && bs->msg[0] == 'U') {        /* SD sending attributes */
@@ -303,7 +302,7 @@ int bget_dirmsg(JCR *jcr, BSOCK *bs, BSOCK_CLIENT_TYPE role)
          continue;
       }
       /* Get Progress: files, bytes, bytes/sec */
-      if (role==BSOCK_TYPE_FD && bs->msg[0] == 'P') {       /* Progress report */
+      if (bs->msg[0] == 'P') {       /* Progress report */
          uint32_t files, bps;
          uint64_t bytes;
          if ((sscanf(bs->msg, "Progress JobId=%ld files=%ld bytes=%lld bps=%ld\n",
@@ -425,12 +424,12 @@ bool response(JCR *jcr, BSOCK *bs, BSOCK_CLIENT_TYPE role, const char *resp, con
          return true;
       }
       if (prtmsg == DISPLAY_ERROR) {
-         Jmsg(jcr, M_FATAL, 0, _("Bad response to %s command: wanted %s, got %s\n"),
+         Jmsg(jcr, M_FATAL, 0, _("[DE0031] Bad response to %s command: wanted %s, got %s\n"),
             cmd, resp, bs->msg);
       }
       return false;
    }
-   Jmsg(jcr, M_FATAL, 0, _("Socket error on %s command: ERR=%s\n"),
+   Jmsg(jcr, M_FATAL, 0, _("[DE0038] Socket error on %s command: ERR=%s\n"),
          cmd, bs->bstrerror());
    return false;
 }
