@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -27,8 +27,8 @@ extern void admin_cleanup(JCR *jcr, int TermCode);
 
 
 /* authenticate.c */
-extern bool authenticate_storage_daemon(JCR *jcr, STORE *store);
-extern int authenticate_file_daemon(JCR *jcr);
+extern bool authenticate_storage_daemon(JCR *jcr, STORE *store, int *status, POOLMEM **errmsg);
+extern int authenticate_file_daemon(JCR *jcr, int *status, POOLMEM **errmsg);
 extern int authenticate_user_agent(UAContext *ua);
 
 /* autoprune.c */
@@ -76,9 +76,11 @@ RBSR_FINDEX *new_findex();
 void make_unique_restore_filename(UAContext *ua, POOLMEM **fname);
 void print_bsr(UAContext *ua, RESTORE_CTX &rx);
 void scan_bsr(JCR *jcr);
+bool split_bsr_loop(JCR *jcr, bootstrap_info &info);
 
 
 /* catreq.c */
+extern void fd_catreq(JCR *jcr, BSOCK *bs);
 extern void catalog_request(JCR *jcr, BSOCK *bs);
 extern void catalog_update(JCR *jcr, BSOCK *bs);
 extern bool despool_attributes_from_file(JCR *jcr, const char *file);
@@ -96,7 +98,7 @@ int variable_expansion(JCR *jcr, char *inp, POOLMEM **exp);
 /* fd_cmds.c */
 extern int connect_to_file_daemon(JCR *jcr, int retry_interval,
                                   int max_retry_time, int verbose);
-extern bool send_ls_fileset(JCR *jcr, const char *path);
+extern bool send_ls_fileset(JCR *jcr, const char *path, bool dironly);
 extern bool send_ls_plugin_fileset(JCR *jcr, const char *plugin, const char *path);
 extern bool send_include_list(JCR *jcr);
 extern bool send_exclude_list(JCR *jcr);
@@ -120,11 +122,13 @@ enum e_prtmsg {
 extern bool response(JCR *jcr, BSOCK *fd, BSOCK_CLIENT_TYPE role, const char *resp, const char *cmd, e_prtmsg prtmsg);
 
 /* job.c */
+extern void jmsg_large_jobid_list(JCR *jcr, const char *msg, const char *jobids);
 extern bool allow_duplicate_job(JCR *jcr);
 extern void set_jcr_defaults(JCR *jcr, JOB *job);
 extern void create_unique_job_name(JCR *jcr, const char *base_name);
 extern void update_job_end_record(JCR *jcr);
 extern bool get_or_create_client_record(JCR *jcr);
+extern void fileset_get_content(FILESET_DBR *fdbr, FILESET *fileset);
 extern bool get_or_create_fileset_record(JCR *jcr);
 extern DBId_t get_or_create_pool_record(JCR *jcr, char *pool_name);
 extern void apply_pool_overrides(JCR *jcr);
@@ -149,6 +153,7 @@ extern bool run_console_command(JCR *jcr, const char *cmd);
 extern void sd_msg_thread_send_signal(JCR *jcr, int sig);
 void terminate_sd_msg_chan_thread(JCR *jcr);
 bool flush_file_records(JCR *jcr);
+void dir_close_batch_connection(JCR *jcr);
 
 /* jobq.c */
 extern bool inc_read_store(JCR *jcr);
@@ -182,7 +187,8 @@ bool has_quota_reached(JCR *jcr, MEDIA_DBR *mr);
 bool has_quota_reached(JCR *jcr);
 void set_storageid_in_mr(STORE *store, MEDIA_DBR *mr);
 int find_next_volume_for_append(JCR *jcr, MEDIA_DBR *mr, int index,
-                                bool create, bool purge, POOL_MEM &errmsg);
+                                bool create, bool purge, int use_protect,
+                                int vol_encrypted, POOL_MEM &errmsg);
 bool has_volume_expired(JCR *jcr, MEDIA_DBR *mr);
 void check_if_volume_valid_or_recyclable(JCR *jcr, MEDIA_DBR *mr, const char **reason);
 bool get_scratch_volume(JCR *jcr, bool InChanger, MEDIA_DBR *mr,
@@ -317,6 +323,7 @@ int scan_truncate_cmd(UAContext *ua, const char *cmd,
                      char *truncate_opt);
 /* ua_status.c */
 void list_dir_status_header(UAContext *ua);
+char *dir_get_information(JCR *jcr, POOLMEM **mem);
 
 /* ua_tree.c */
 bool user_select_files_from_tree(TREE_CTX *tree);
@@ -367,6 +374,9 @@ bool update_snapshot(UAContext *ua);
 int prune_snapshot(UAContext *ua);
 bool send_snapshot_retention(JCR *jcr, utime_t val);
 
+/* dird.c */
+bool reload_config(int sig, alist *msglist);
+
 /* ua_collect.c */
 bool update_permanent_stats(void *data);
 void initialize_statcollector();
@@ -387,6 +397,9 @@ bool catreq_get_pool_info(JCR *jcr, BSOCK *bs);
 bool is_dedup_ref(DEV_RECORD *rec, bool lazy);
 
 /* dir_authplugins.c */
-void *dir_authplugin_getauthenticationData(JCR *jcr, const char *param);
+void * dir_authplugin_getauthenticationData(JCR *jcr, const char *console, const char *param);
 bRC dir_authplugin_do_interaction(JCR *jcr, BSOCK *bsock, const char *pluginname, void *data, bool pluginall = false);
 bRC dir_authplugin_authenticate(JCR *jcr, BSOCK *bsock, const char *pluginname);
+
+/* malware.c */
+int check_malware(JCR *jcr, const char *jobids, POOLMEM **errmsg);
