@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -40,10 +40,10 @@ extern struct s_jt jobtypes[];
 /* Imported functions */
 extern void do_messages(UAContext *ua, const char *cmd);
 extern int quit_cmd(UAContext *ua, const char *cmd);
-extern int qhelp_cmd(UAContext *ua, const char *cmd);
 extern bool dot_status_cmd(UAContext *ua, const char *cmd);
 extern void bvfs_set_acl(UAContext *ua, Bvfs *bvfs);
 extern bool jlist_cmd(UAContext *ua, const char *cmd);
+extern int qhelp_cmd(UAContext *ua, const char *cmd);
 
 /* Forward referenced functions */
 static bool admin_cmds(UAContext *ua, const char *cmd);
@@ -85,7 +85,7 @@ static bool dot_bvfs_get_bootstrap(UAContext *ua, const char *cmd);
 static bool dot_bvfs_get_fileindex(UAContext *ua, const char *cmd);
 static bool dot_bvfs_get_delta(UAContext *ua, const char *cmd);
 static bool dot_bvfs_delete_fileid(UAContext *ua, const char *cmd);
-static void bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len);
+static bool bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len);
 static bool dot_setuid(UAContext *ua, const char *cmd);
 static bool dot_querycmd(UAContext *ua, const char *cmd);
 static bool putfile_cmd(UAContext *ua, const char *cmd);
@@ -95,59 +95,72 @@ static bool dot_quit_cmd(UAContext *ua, const char *cmd);
 static bool dot_help_cmd(UAContext *ua, const char *cmd);
 static bool dot_add_events(UAContext *ua, const char *cmd);
 static int one_handler(void *ctx, int num_field, char **row);
+static bool dot_search(UAContext *ua, const char *cmd);
 
-struct dcmd_struct { const char *key; bool (*func)(UAContext *ua, const char *cmd); const char *help;const bool use_in_rs;};
+struct dcmd_struct { const char *key; bool (*func)(UAContext *ua, const char *cmd); const char *help; const char *usage;const bool use_in_rs;};
 static struct dcmd_struct commands[] = { /* help */  /* can be used in runscript */
- { NT_(".api"),        api_cmd,                  NULL,       false},
- { NT_(".backups"),    backupscmd,               NULL,       false},
- { NT_(".clients"),    clientscmd,               NULL,       true},
- { NT_(".catalogs"),   catalogscmd,              NULL,       false},
- { NT_(".defaults"),   defaultscmd,              NULL,       false},
- { NT_(".die"),        admin_cmds,               NULL,       false},
- { NT_(".dump"),       admin_cmds,               NULL,       false},
- { NT_(".exit"),       admin_cmds,               NULL,       false},
- { NT_(".events"),     dot_add_events,           NULL,       false},
- { NT_(".filesets"),   filesetscmd,              NULL,       false},
- { NT_(".help"),       dot_help_cmd,             NULL,       false},
- { NT_(".jobs"),       jobscmd,                  NULL,       true},
- { NT_(".estimate"),   dotestimatecmd,           NULL,       false},
- { NT_(".jlist"),      jlist_cmd,                NULL,       false},
- { NT_(".levels"),     levelscmd,                NULL,       false},
- { NT_(".messages"),   getmsgscmd,               NULL,       false},
- { NT_(".msgs"),       msgscmd,                  NULL,       false},
- { NT_(".pools"),      poolscmd,                 NULL,       true},
- { NT_(".quit"),       dot_quit_cmd,             NULL,       false},
- { NT_(".putfile"),    putfile_cmd,              NULL,       false}, /* use @putfile */
- { NT_(".schedule"),   schedulescmd,             NULL,       false},
- { NT_(".sql"),        sql_cmd,                  NULL,       false},
- { NT_(".status"),     dot_status_cmd,           NULL,       false},
- { NT_(".storage"),    storagecmd,               NULL,       true},
- { NT_(".volstatus"),  volstatuscmd,             NULL,       true},
- { NT_(".media"),      mediacmd,                 NULL,       true},
- { NT_(".mediatypes"), mediatypescmd,            NULL,       true},
- { NT_(".locations"),  locationscmd,             NULL,       true},
- { NT_(".actiononpurge"),aopcmd,                 NULL,       true},
- { NT_(".bvfs_lsdirs"), dot_bvfs_lsdirs,         NULL,       true},
- { NT_(".bvfs_lsfiles"),dot_bvfs_lsfiles,        NULL,       true},
- { NT_(".bvfs_get_volumes"),dot_bvfs_get_volumes,NULL,       true},
- { NT_(".bvfs_update"), dot_bvfs_update,         NULL,       true},
- { NT_(".bvfs_get_jobids"), dot_bvfs_get_jobids, NULL,       true},
- { NT_(".bvfs_get_jobs"), dot_bvfs_get_jobs,     NULL,       true},
- { NT_(".bvfs_get_bootstrap"), dot_bvfs_get_bootstrap,NULL,  true},
- { NT_(".bvfs_get_fileindex"), dot_bvfs_get_fileindex,NULL,  true},
- { NT_(".bvfs_versions"), dot_bvfs_versions,     NULL,       true},
- { NT_(".bvfs_get_delta"), dot_bvfs_get_delta,   NULL,       true},
- { NT_(".bvfs_restore"),  dot_bvfs_restore,      NULL,       true},
- { NT_(".bvfs_cleanup"),  dot_bvfs_cleanup,      NULL,       true},
- { NT_(".bvfs_decode_lstat"),dot_bvfs_decode_lstat,NULL,     true},
- { NT_(".bvfs_clear_cache"),dot_bvfs_clear_cache,NULL,       false},
- { NT_(".bvfs_update_fv"),dot_bvfs_update_fv,    NULL,       true},
- { NT_(".bvfs_delete_fileid"),dot_bvfs_delete_fileid,  NULL, false},
- { NT_(".setuid"), dot_setuid,                   NULL,       false},
- { NT_(".ls"), dot_ls_cmd,                       NULL,       false},
- { NT_(".types"),      typescmd,                 NULL,       false},
- { NT_(".query"),     dot_querycmd,              NULL,       false},
- { NT_(".tags"),      tagscmd,                   NULL,       false}
+   { NT_(".api"),        api_cmd,        _("Set the API options"),  ".api 2\n.api 2 api_opts=j",             false},
+   { NT_(".backups"),    backupscmd,     _("Display the list of the authorized Backup Job resources"), NULL, false},
+   { NT_(".clients"),    clientscmd,     _("Display the list of the authorized Client resources"),     NULL, true},
+   { NT_(".catalogs"),   catalogscmd,    _("Display the list of the authorized Catalog resources"),    NULL, false},
+   { NT_(".defaults"),   defaultscmd,    _("Display default values for a Job/Pool/Storage/Client"),
+     ".defaults job=<job>\n"
+     "\t.defaults pool=<pool>\n"
+     "\t.defaults storage=<sto>\n"
+     "\t.defaults client=<cli>",                                    false},
+ { NT_(".die"),        admin_cmds,               NULL,                                                 NULL, false},
+ { NT_(".dump"),       admin_cmds,               NULL,                                                 NULL, false},
+ { NT_(".exit"),       admin_cmds,               _("Exit bconsole"),                                   NULL, false},
+ { NT_(".events"),     dot_add_events,           _("Send event from the console"),
+   ".events source=<txt> type=<type> text=<txt>",               false},
+ { NT_(".filesets"),   filesetscmd,              _("Display the list of the authorized FileSet resources"),      NULL, false},
+ { NT_(".help"),       dot_help_cmd,             _("Display help message on commands"),   ".help .status", false},
+ { NT_(".jobs"),       jobscmd,                  _("Display the list of the authorized Job resources"),          NULL, true},
+ { NT_(".estimate"),   dotestimatecmd,           NULL,                                                           NULL, false},
+ { NT_(".jlist"),      jlist_cmd,                _("JSON friendly version of the list command"),                 NULL, false},
+ { NT_(".levels"),     levelscmd,                _("Display the list of the Job Levels"),                        NULL, false},
+ { NT_(".messages"),   getmsgscmd,               NULL,       NULL, false},
+ { NT_(".msgs"),       msgscmd,                  NULL,       NULL, false},
+ { NT_(".pools"),      poolscmd,                 _("Display the list of the authorized Pool resources"),         NULL, true},
+ { NT_(".quit"),       dot_quit_cmd,             NULL,       NULL, false},
+ { NT_(".putfile"),    putfile_cmd,              NULL,       NULL, false}, /* use @putfile */
+ { NT_(".search"),     dot_search,               _("Search in the catalog"),
+   ".search text=xxx [category=(client|job|volume)]",           false},
+ { NT_(".schedule"),   schedulescmd,             _("Display the list of the authorized Schedule resources"),     NULL, false},
+ { NT_(".sql"),        sql_cmd,                  NULL,       NULL, false},
+ { NT_(".status"),     dot_status_cmd,           _("Query Bacula daemons"),
+   ".status dir [header|running [client=<cli>]|scheduled|terminated|statistics]\n"
+   "\t.status client=<cli> [header|running|terminated]\n"
+   "\t.status storage=<sto> [device=<dev>] [header|running|terminated|collector|devices]", false},
+ { NT_(".storage"),    storagecmd,               _("Display the list of Storage resource defined"),       NULL, true},
+ { NT_(".volstatus"),  volstatuscmd,             _("Display the list of the Volume statuses"),            NULL, true},
+ { NT_(".media"),      mediacmd,                 _("Display the list of the authorized Volumes"),         NULL, true},
+ { NT_(".mediatypes"), mediatypescmd,            _("Display the list of the defined MediaType"),          NULL, true},
+ { NT_(".locations"),  locationscmd,             _("Display the list of the defined Location"),           NULL, true},
+ { NT_(".actiononpurge"),aopcmd,                 _("Display the list of the possible values for ActionOnPurge"),       NULL, true},
+ { NT_(".bvfs_lsdirs"), dot_bvfs_lsdirs,         NULL,       NULL, true},
+ { NT_(".bvfs_lsfiles"),dot_bvfs_lsfiles,        NULL,       NULL, true},
+ { NT_(".bvfs_get_volumes"),dot_bvfs_get_volumes,NULL,       NULL, true},
+ { NT_(".bvfs_update"), dot_bvfs_update,         NULL,       NULL, true},
+ { NT_(".bvfs_get_jobids"), dot_bvfs_get_jobids, NULL,       NULL, true},
+ { NT_(".bvfs_get_jobs"), dot_bvfs_get_jobs,     NULL,       NULL, true},
+ { NT_(".bvfs_get_bootstrap"), dot_bvfs_get_bootstrap,NULL,  NULL, true},
+ { NT_(".bvfs_get_fileindex"), dot_bvfs_get_fileindex,NULL,  NULL, true},
+ { NT_(".bvfs_versions"), dot_bvfs_versions,     NULL,       NULL, true},
+ { NT_(".bvfs_get_delta"), dot_bvfs_get_delta,   NULL,       NULL, true},
+ { NT_(".bvfs_restore"),  dot_bvfs_restore,      NULL,       NULL, true},
+ { NT_(".bvfs_cleanup"),  dot_bvfs_cleanup,      NULL,       NULL, true},
+ { NT_(".bvfs_decode_lstat"),dot_bvfs_decode_lstat,NULL,     NULL, true},
+ { NT_(".bvfs_clear_cache"),dot_bvfs_clear_cache,NULL,       NULL, false},
+ { NT_(".bvfs_update_fv"),dot_bvfs_update_fv,    NULL,       NULL, true},
+ { NT_(".bvfs_delete_fileid"),dot_bvfs_delete_fileid,  NULL, NULL, false},
+ { NT_(".setuid"), dot_setuid,                   _("Set UID/GID for the current session"), ".setuid uid=<uid> gid=<gid>", false},
+ { NT_(".ls"), dot_ls_cmd,                       _("List files/directories on Client"),
+                                                   ".ls client=<cli> path=<str> [dironly] [plugin=<str>]",       false},
+ { NT_(".types"),      typescmd,                 _("List Job Types defined"),        NULL, false},
+ { NT_(".query"),     dot_querycmd,              _("Query Plugin's Client"),
+   ".query client=<cli> plugin=<str> parameter=<str>", false},
+ { NT_(".tags"),      tagscmd,                   _("List debug tags defined"),       NULL, false}
 };
 #define comsize ((int)(sizeof(commands)/sizeof(struct dcmd_struct)))
 
@@ -208,7 +221,10 @@ bool do_a_dot_command(UAContext *ua)
    return ok;
 }
 
-/* Store the current uid/gid restriction */
+/*
+ * .setuid uid=<uid> gid=<gid>
+ * Store the current uid/gid restriction for the current bconsole session
+ */
 static bool dot_setuid(UAContext *ua, const char *cmd)
 {
    ua->uid = 0;
@@ -232,7 +248,8 @@ static bool dot_setuid(UAContext *ua, const char *cmd)
 }
 
 /*
- * Send ls to Client
+ * .ls client=<cli> [plugin=<plugin>|path=<path>]
+ * Send ls to Client to list files and directories
  */
 static bool dot_ls_cmd(UAContext *ua, const char *cmd)
 {
@@ -242,6 +259,8 @@ static bool dot_ls_cmd(UAContext *ua, const char *cmd)
    char *plugin = NULL;
    JCR *jcr = ua->jcr;
    int i;
+   bool ret = false;
+   bool dironly = false;
 
    jcr->setJobLevel(L_FULL);
    i = find_arg_with_value(ua, NT_("client"));
@@ -270,6 +289,11 @@ static bool dot_ls_cmd(UAContext *ua, const char *cmd)
       return false;
    }
 
+   i = find_arg(ua, NT_("dironly"));
+   if (i > 0) {
+      dironly = true;
+   }
+
    /* optional plugin=... parameter */
    i = find_arg_with_value(ua, NT_("plugin"));
    if (i > 0) {
@@ -287,8 +311,8 @@ static bool dot_ls_cmd(UAContext *ua, const char *cmd)
                 jcr->client->FDport);
 
    if (!connect_to_file_daemon(jcr, 1, 15, 0)) {
-      ua->error_msg(_("Failed to connect to Client.\n"));
-      return false;
+      ua->error_msg("%s", jcr->errmsg);
+      goto bail_out;
    }
 
    /* when .ls plugin prepare a special ls_plugin_fileset */
@@ -298,7 +322,7 @@ static bool dot_ls_cmd(UAContext *ua, const char *cmd)
          goto bail_out;
       }
    } else {
-      if (!send_ls_fileset(jcr, path)) {
+      if (!send_ls_fileset(jcr, path, dironly)) {
          ua->error_msg(_("Failed to send command to Client.\n"));
          goto bail_out;
       }
@@ -309,12 +333,15 @@ static bool dot_ls_cmd(UAContext *ua, const char *cmd)
       ua->send_msg("%s", jcr->file_bsock->msg);
    }
 
+   ret = true;
+
 bail_out:
    if (jcr->file_bsock) {
       jcr->file_bsock->signal(BNET_TERMINATE);
       free_bsock(ua->jcr->file_bsock);
    }
-   return true;
+   jcr->client = NULL;
+   return ret;
 }
 
 #ifdef COMMUNITY
@@ -323,6 +350,10 @@ void bvfs_set_acl(UAContext *ua, Bvfs *bvfs)
 }
 #endif
 
+/*
+ * .bvfs_decode_lstat lstat="AAAA AAAA AAAA AAAA"
+ * Decode the Lstat database field
+ */
 static bool dot_bvfs_decode_lstat(UAContext *ua, const char *cmd)
 {
    int32_t LinkFI;
@@ -492,10 +523,10 @@ static int bvfs_result_handler(void *ctx, int fields, char **row)
                    jobid, lstat, path);
 
    } else if (bvfs_is_version(row)) {
-      ua->send_msg("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", row[BVFS_PathId],
+      ua->send_msg("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", row[BVFS_PathId],
                    row[BVFS_FilenameId], fileid, jobid,
                    lstat, row[BVFS_Md5], row[BVFS_VolName],
-                   row[BVFS_VolInchanger]);
+                   row[BVFS_VolInchanger], row[BVFS_VolType]);
 
    } else if (bvfs_is_file(row)) {
       ua->send_msg("%s\t%s\t%s\t%s\t%s\t%s\n", row[BVFS_PathId],
@@ -503,8 +534,8 @@ static int bvfs_result_handler(void *ctx, int fields, char **row)
                    lstat, row[BVFS_Name]);
 
    } else if (bvfs_is_volume_list(row)) {
-      ua->send_msg("%s\t%s\n", row[BVFS_VolName],
-                   row[BVFS_VolInchanger]);
+      ua->send_msg("%s\t%s\t%s\n", row[BVFS_VolName],
+                   row[BVFS_VolInchanger], row[BVFS_VolType]);
 
    } else if (bvfs_is_delta_list(row)) {
       ua->send_msg("%s\t%s\t%s\t%s\t%s\t%s\t%s\n", row[BVFS_PathId],
@@ -541,10 +572,16 @@ static bool bvfs_parse_arg_version(UAContext *ua,
                                    bool *copies)
 {
    bool fnid_found=false;
-   *fnid=0;
    *client=NULL;
-   *versions=false;
-   *copies=false;
+   if (copies) {
+      *copies=false;
+   }
+   if (fnid) {
+      *fnid=0;
+   }
+   if (versions) {
+      *versions=false;
+   }
 
    for (int i=1; i<ua->argc; i++) {
       if (fnid && strcasecmp(ua->argk[i], NT_("fnid")) == 0) {
@@ -672,7 +709,7 @@ static int get_client_jobids(UAContext *ua, Bvfs *fs)
    char limit[50];
    alist clients(10, owned_by_alist);
    db_list_ctx jobids;
-   bool ret;
+   bool ret, use_all_jobtype=false;
 
    int pos = find_arg_with_value(ua, "client");
    if (pos < 0) {
@@ -686,14 +723,15 @@ static int get_client_jobids(UAContext *ua, Bvfs *fs)
    parse_list(ua->argv[pos], &clients);
 
    db_lock(ua->db);
-   bvfs_get_filter(ua, where, limit, sizeof(limit));
+   use_all_jobtype = bvfs_get_filter(ua, where, limit, sizeof(limit));
    Mmsg(ua->db->cmd,
       "SELECT JobId "
         "FROM Job JOIN Client USING (ClientId) "
          "WHERE Client.Name IN (%s) "
-           "AND Job.Type = 'B' AND Job.JobStatus IN ('T', 'W') %s "
+           "AND %s AND Job.JobStatus IN ('T', 'W') %s "
          "ORDER By JobTDate ASC %s",
            fs->escape_list(&clients),
+           use_all_jobtype ? " Job.Type IN ('B', 'C', 'M') " : " Job.Type = 'B' ",
            where.c_str(), limit);
    ret = db_sql_query(ua->db, ua->db->cmd, db_list_handler, &jobids);
    db_unlock(ua->db);
@@ -820,13 +858,13 @@ bail_out:
    return ret;
 }
 
-/* .bvfs_restore path=b2XXXXX jobid=1,2 fileid=1,2 dirid=1,2 objectid=1,2
+/* .bvfs_restore path=b2XXXXX jobid=1,2 fileid=1,2 dirid=1,2 fileindex=jobid,fidx,jobid,fidx objectid=1,2
  */
 static bool dot_bvfs_restore(UAContext *ua, const char *cmd)
 {
    DBId_t pathid=0;
    int limit=2000, offset=0, i;
-   char *path=NULL, *jobid = NULL, *username=NULL;
+   char *path=NULL, *jobid = NULL, *username=NULL, *fileindex=NULL;
    db_list_ctx jobids, fileid, dirid;
    bool object = false, ret = false;
 
@@ -861,6 +899,7 @@ static bool dot_bvfs_restore(UAContext *ua, const char *cmd)
       }
       fileid.add(ua->argv[i]);
    }
+
    if ((i = find_arg_with_value(ua, "dirid")) >= 0) {
       if (ua->argv[i][0] != '\0' && !is_a_number_list(ua->argv[i])) {
          ua->error_msg("Please provide dirid as a list of integers!\n");
@@ -869,6 +908,27 @@ static bool dot_bvfs_restore(UAContext *ua, const char *cmd)
       dirid.add(ua->argv[i]);
    }
 
+   if ((i = find_arg_with_value(ua, "fileindex")) >= 0) {
+      if (ua->argv[i][0] != '\0' && !is_a_number_list(ua->argv[i])) {
+         ua->error_msg("Please provide fileid as a list of integers!\n");
+         return true;
+      }
+
+      int64_t jobid, fidx;
+      sellist sel;
+      sel.set_string(ua->argv[i], true);
+
+      /* check the format */
+      foreach_sellist(jobid, &sel) {
+         fidx = sel.next();
+         if (fidx <= 0) {
+            ua->error_msg("Please provide jobid,fileindex as a list of integers!\n");
+            return true;
+         }
+      }
+      fileindex = ua->argv[i];
+   }
+
    if (object) {
       i = find_arg_with_value(ua, "objectid");
       if (ua->argv[i][0] != '\0' && !is_a_number_list(ua->argv[i])) {
@@ -893,7 +953,7 @@ static bool dot_bvfs_restore(UAContext *ua, const char *cmd)
    if ((i = find_arg(ua, "nodelta")) >= 0) {
       fs.set_compute_delta(false);
    }
-   if (fs.compute_restore_list(fileid.list, dirid.list, path)) {
+   if (fs.compute_restore_list(fileid.list, dirid.list, fileindex, path)) {
       ua->send_msg("OK\n");
    } else {
       ua->error_msg("Can't create restore list\n");
@@ -1585,12 +1645,17 @@ static char *get_argument(UAContext *ua, const char *arg, char *esc, bool conver
    return esc;
 }
 
-/* The DB should be locked */
-static void bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len)
+/* The DB should be locked 
+ * Return:
+ * true: Can select all kind of job type (M, C, B) (we have  a job or a jobid)
+ * false: Filter on Backup jobs
+ */
+static bool bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len)
 {
    POOL_MEM tmp;
    char esc_name[MAX_ESCAPE_NAME_LENGTH];
-
+   bool ret = false;
+   
    if (get_argument(ua, "jobname", esc_name, true) != NULL) {
       Mmsg(where, "AND Job.Job LIKE '%s' ", esc_name);
    }
@@ -1605,12 +1670,14 @@ static void bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len
       if (strcmp(esc_name, "0") != 0) {
          Mmsg(tmp, "AND Job.JobId = '%s' ", esc_name);
          pm_strcat(where, tmp.c_str());
+         ret = true;
       }
    }
 
    if (get_argument(ua, "ujobid", esc_name, false) != NULL) {
       Mmsg(tmp, "AND Job.Job = '%s' ", esc_name);
       pm_strcat(where, tmp.c_str());
+      ret = true;
    }
 
    if (get_argument(ua, "start", esc_name, false) != NULL) {
@@ -1629,6 +1696,7 @@ static void bvfs_get_filter(UAContext *ua, POOL_MEM &where, char *limit, int len
          bsnprintf(limit, len, "LIMIT %s ", esc_name);
       }
    }
+   return ret;
 }
 
 /* .bvfs_get_jobs client=xxx [ujobid=yyyy] [jobname=<glob>] [fileset=<glob>] [start=<ts>] [end=<ts>]
@@ -1641,6 +1709,8 @@ static bool dot_bvfs_get_jobs(UAContext *ua, const char *cmd)
    POOL_MEM where;
    char esc_cli[MAX_ESCAPE_NAME_LENGTH];
    char limit[MAX_ESCAPE_NAME_LENGTH];
+   bool use_all_jobtype=false;
+
    if (!open_new_client_db(ua)) {
       return true;
    }
@@ -1660,15 +1730,17 @@ static bool dot_bvfs_get_jobs(UAContext *ua, const char *cmd)
    db_escape_string(ua->jcr, ua->db, esc_cli,
                     ua->argv[pos], strlen(ua->argv[pos]));
 
-   bvfs_get_filter(ua, where, limit, sizeof(limit));
+   use_all_jobtype = bvfs_get_filter(ua, where, limit, sizeof(limit));
 
    Mmsg(ua->db->cmd,
         "SELECT JobId, JobTDate, HasCache, Job "
           "FROM Job JOIN Client USING (ClientId) JOIN FileSet USING (FileSetId) "
-         "WHERE Client.Name = '%s' AND Job.Type = 'B' AND Job.JobStatus IN ('T', 'W') "
+         "WHERE Client.Name = '%s' AND %s AND Job.JobStatus IN ('T', 'W') "
             "%s "
          "ORDER By JobTDate DESC %s",
-        esc_cli, where.c_str(), limit);
+        esc_cli,
+        use_all_jobtype ? " Job.Type IN ('B', 'C', 'M') " : " Job.Type = 'B' ",
+        where.c_str(), limit);
 
    db_sql_query(ua->db, ua->db->cmd, jobs_handler, ua);
    db_unlock(ua->db);
@@ -1683,7 +1755,47 @@ static bool dot_quit_cmd(UAContext *ua, const char *cmd)
 
 static bool dot_help_cmd(UAContext *ua, const char *cmd)
 {
-   qhelp_cmd(ua, cmd);
+   int i;
+   const char *msg="";
+   const char *kw = ua->argk[1];
+
+   if (ua->argc == 2) {
+      /* Specific case for Tab completion */
+      if (strcasecmp(ua->argk[1], NT_("item")) == 0) {
+         return qhelp_cmd(ua, cmd);
+      }
+      /* Specific case for Tab completion */
+      if (strcasecmp(ua->argk[1], NT_("all")) == 0) {
+         return qhelp_cmd(ua, cmd);
+      }
+      if (strcasecmp(ua->argk[0], NT_(".help")) == 0) {
+         kw = ua->argk[1];
+
+      } else {
+         kw = ua->argk[0];
+      }
+   }
+   ua->send_msg(_("  Command       Description\n  =======       ===========\n"));
+   for (i=0; i<comsize; i++) {
+      if (ua->argc == 2) {
+         if (!strcasecmp(kw, commands[i].key)) {
+            ua->send_msg(_("  %-13s %s\n\nArguments:\n\t%s\n"), commands[i].key,
+                         NPRTB(commands[i].help), NPRTB(commands[i].usage));
+            break;
+         }
+      } else {
+         if (acl_access_ok(ua, Command_ACL, commands[i].key, strlen(commands[i].key))) {
+            ua->send_msg(_(" %-13s %s\n"),
+                         NPRTB(commands[i].key), NPRTB(commands[i].help));
+         } else {
+            msg = _("Some commands are not available in this restricted console. ");
+         }
+      }
+   }
+   if (i == comsize && ua->argc == 2) {
+      ua->send_msg(_("\nCan't find %s command.\n\n"), kw);
+   }
+   ua->send_msg(_("\n%sWhen at a prompt, entering a period cancels the command.\n\n"), msg);
    return true;
 }
 
@@ -1706,8 +1818,8 @@ static void do_storage_cmd(UAContext *ua, STORE *store, const char *cmd)
    ua->send_msg(_("Connecting to Storage daemon %s at %s:%d\n"),
       store->name(), store->address, store->SDport);
    if (!connect_to_storage_daemon(jcr, 1, 15, 0)) {
-      ua->error_msg(_("Failed to connect to Storage daemon.\n"));
-      return;
+      ua->error_msg("%s", jcr->errmsg);
+      goto bail_out;
    }
    Dmsg0(120, _("Connected to storage daemon\n"));
    sd = jcr->store_bsock;
@@ -1716,6 +1828,8 @@ static void do_storage_cmd(UAContext *ua, STORE *store, const char *cmd)
       ua->send_msg("%s", sd->msg);
    }
    sd->signal(BNET_TERMINATE);
+
+bail_out:
    free_bsock(ua->jcr->store_bsock);
    return;
 }
@@ -1731,7 +1845,7 @@ static void do_client_cmd(UAContext *ua, CLIENT *client, const char *cmd)
    ua->send_msg(_("Connecting to Client %s at %s:%d\n"),
                 client->name(), get_client_address(ua->jcr, client, buf.addr()), client->FDport);
    if (!connect_to_file_daemon(ua->jcr, 1, 15, 0)) {
-      ua->error_msg(_("Failed to connect to Client.\n"));
+      ua->error_msg("%s", ua->jcr->errmsg);
       goto bail_out;
    }
    Dmsg0(120, "Connected to file daemon\n");
@@ -1744,6 +1858,7 @@ static void do_client_cmd(UAContext *ua, CLIENT *client, const char *cmd)
 
 bail_out:
    free_bsock(ua->jcr->file_bsock);
+   ua->jcr->client = NULL;
    return;
 }
 
@@ -2347,7 +2462,7 @@ static bool dot_querycmd(UAContext *ua, const char *cmd)
    init_jcr_job_record(jcr);           // need job
 
    if (!connect_to_file_daemon(jcr, 1, 15, 0)) {
-      ua->error_msg(_("error=Failed to connect to Client.\n"));
+      ua->error_msg("%s", jcr->errmsg);
       goto bail_out;
    }
 
@@ -2362,6 +2477,7 @@ static bool dot_querycmd(UAContext *ua, const char *cmd)
    while (jcr->file_bsock->recv() >= 0) {
       ua->send_msg("%s", jcr->file_bsock->msg);
    }
+   ret = true;
 
    ret = true;
 
@@ -2578,7 +2694,7 @@ static bool levelscmd(UAContext *ua, const char *cmd)
 static bool volstatuscmd(UAContext *ua, const char *cmd)
 {
    ua->send_msg("Append\n");
-   ua->send_msg("Archived\n");
+   ua->send_msg("Archive\n");
    ua->send_msg("Disabled\n");
    ua->send_msg("Full\n");
    ua->send_msg("Used\n");
@@ -2737,3 +2853,188 @@ static bool dot_add_events(UAContext *ua, const char *cmd)
 bail_out:
    return true;
 }
+
+enum {
+   CAT_VOLUME = (1<<0),
+   CAT_JOB    = (1<<1),
+   CAT_CLIENT = (1<<2),
+};
+
+#define CAT_ALL (CAT_VOLUME | CAT_JOB | CAT_CLIENT)
+
+/* Search over our different tables
+ *
+ * text=xxx [category=client|job|volume]
+ * {
+ *   "error": 0,
+ *   "errmsg", "",
+ *   "type": "search",
+ *   "data": {
+ *      "volume": ["vol1", "vol2"],
+ *      "client": ["volA"],
+ *      "job": []
+ *   }
+ * }
+ *
+ */
+static bool dot_search(UAContext *ua, const char *cmd)
+{
+   uint32_t cat = 0;
+   uint64_t limit=0;
+   char *text = NULL;
+   const char *errmsg = NULL;
+   alist res(10, owned_by_alist);
+   OutputWriter out(ua->api_opts);
+   alist *r = &res;
+
+   for(int i=1; i < ua->argc ; i++) {
+      if (strcasecmp(ua->argk[i], NT_("limit")) == 0 && is_a_number(ua->argv[i])) {
+         limit = str_to_uint64(ua->argv[i]);
+
+      } else if (strcasecmp(ua->argk[i], NT_("category")) == 0) {
+         if (strcasecmp(NPRTB(ua->argv[i]), "all") == 0) {
+            cat = CAT_ALL;
+
+         } else if (strcasecmp(NPRTB(ua->argv[i]), "client") == 0) {
+            cat |= CAT_CLIENT;
+
+         } else if (strcasecmp(NPRTB(ua->argv[i]), "job") == 0) {
+            cat |= CAT_JOB;
+
+         } else if (strcasecmp(NPRTB(ua->argv[i]), "volume") == 0) {
+            cat |= CAT_VOLUME;
+
+         } else {
+            errmsg = "Incorrect category";
+            goto bail_out;
+         }
+
+      } else if (strcasecmp(ua->argk[i], NT_("text")) == 0) {
+         text = ua->argv[i];
+
+      } else {
+         errmsg = "Incorrect parameter";
+         goto bail_out;
+      }
+   }
+   if (!text || strlen(text) <= 3) {
+      errmsg = "Missing text to search";
+      goto bail_out;
+   }
+
+   /* TODO: check for nasty characters in text */
+   if (!cat) {
+      cat = CAT_ALL;
+   }
+   if (!limit) {
+      limit = 10;
+   }
+   if (!open_new_client_db(ua)) {
+      errmsg = "Unable to open the catalog";
+      goto bail_out;
+   }
+
+   /* We initialize the output */
+   out.get_output(OT_START_OBJ,
+                  OT_INT,    "error", 0,
+                  OT_STRING, "errmsg", "",
+                  OT_STRING, "type", "search",
+                  OT_END);
+   out.start_object("data");
+
+   if (text[0] == '#') {
+      /* search for a tag */
+      TAG_DBR tr;
+      r->destroy();
+      if (cat & CAT_CLIENT) {
+         tr.zero();
+         tr.all = true;
+         bstrncpy(tr.Name, text, sizeof(tr.Name));
+         strcpy(tr.Client, "1"); // To enable the Client search
+         if (!db_search_tag_records(ua->jcr, ua->db, &tr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query client tag";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "client", r, OT_END);
+
+      r->destroy();
+      if (cat  & CAT_JOB) {
+         tr.zero();
+         tr.all = true;
+         bstrncpy(tr.Name, text, sizeof(tr.Name));
+         strcpy(tr.Job, "1"); // To enable the Job search
+         if (!db_search_tag_records(ua->jcr, ua->db, &tr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query Job tag";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "job", r, OT_END);
+
+      r->destroy();
+      if (cat & CAT_VOLUME) {
+         tr.zero();
+         tr.all = true;
+         bstrncpy(tr.Name, text, sizeof(tr.Name));
+         strcpy(tr.Volume, "1"); // To enable the Volume search
+         if (!db_search_tag_records(ua->jcr, ua->db, &tr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query Volume tag";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "volume", r, OT_END);
+
+   } else {
+      r->destroy();
+      if (cat & CAT_VOLUME) {
+         /* Search in volumes */
+         MEDIA_DBR mdbr;
+         bstrncpy(mdbr.VolumeName, text, sizeof(mdbr.VolumeName));
+         mdbr.limit = limit;
+         if (!db_search_media_records(ua->jcr, ua->db, &mdbr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query Volume";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "volume", r, OT_END);
+
+      /* Search in clients */
+      r->destroy();
+      if (cat & CAT_CLIENT) {
+         CLIENT_DBR cr;
+         memset(&cr, 0, sizeof(cr));
+         cr.limit = limit;
+         bstrncpy(cr.Name, text, sizeof(cr.Name));
+         if (!db_search_client_records(ua->jcr, ua->db, &cr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query Client";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "client", &res, OT_END);
+
+      /* Search in Job */
+      r->destroy();
+      if (cat & CAT_JOB) {
+         JOB_DBR jr;
+         memset(&jr, 0, sizeof(jr));
+         jr.limit = limit;
+         bstrncpy(jr.Job, text, sizeof(jr.Job));
+         if (!db_search_job_records(ua->jcr, ua->db, &jr, db_string_list_handler, &r)) {
+            errmsg = "Unable to query Job";
+            goto bail_out;
+         }
+      }
+      out.get_output(OT_ALIST_STR, "job", &res, OT_END);
+   }
+   out.end_object();
+   ua->send_msg(out.get_output(OT_END_OBJ, OT_END));
+   return true;
+
+bail_out:
+   ua->send_msg(out.get_output(OT_CLEAR,
+                               OT_START_OBJ,
+                               OT_INT,    "error",  1,
+                               OT_STRING, "errmsg", errmsg,
+                               OT_END_OBJ, OT_END));
+   return true;
+}
