@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -173,11 +173,11 @@ int run_cmd(UAContext *ua, const char *cmd)
        *   allow him to modify them.
        */
       if (!display_job_parameters(ua, jcr, rc.job, rc.verify_list, rc.jid, rc.replace,
-           rc.client ? rc.client->hdr.name : jcr->job->client->hdr.name)) {
+           jcr->job->client->hdr.name)) {
          break; /* error get out of while loop */
       }
 
-      if (!get_cmd(ua, _("OK to run? (yes/mod/no): "))) {
+      if (!get_cmd(ua, _("OK to run? (Yes/mod/no): "))) {
          break; /* error get out of while loop */
       }
 
@@ -970,8 +970,8 @@ static int plugin_display_options(UAContext *ua, JCR *jcr, ConfigFile *ini)
 
 configure_again:
    ua->send_msg(_("Plugin Restore Options\n"));
-   ua->send_msg(_("Option               Current Value        Default Value\n"));
-   for (nb=0; ini->items[nb].name; nb++) {
+   ua->send_msg(_("Option                         Current Value        Default Value\n"));
+   for (nb=0; nb < MAX_INI_ITEMS && ini->items[nb].name ; nb++) {
 
       if (ini->items[nb].found) {
          /* When calling the handler, It will convert the value
@@ -989,7 +989,7 @@ configure_again:
 
       Mmsg(tmp, "%s:", ini->items[nb].name);
 
-      Mmsg(prompt, "%-20s %-20s ",
+      Mmsg(prompt, "%-30s %-20s ",
            tmp.c_str(), ini->edit);
 
       if (ini->items[nb].default_value) {
@@ -1000,7 +1000,9 @@ configure_again:
       ua->send_msg("%s\n", prompt.c_str());
    }
 
-   if (!get_cmd(ua, _("Use above plugin configuration? (yes/mod/no): "))) {
+   ASSERTD(nb < MAX_INI_ITEMS, "Check if we reach the MAX_INI_ITEMS defined");
+
+   if (!get_cmd(ua, _("Use above plugin configuration? (Yes/mod/no): "))) {
       ini->clear_items();
       return 0;
    }
@@ -1223,8 +1225,8 @@ int modify_job_parameters(UAContext *ua, JCR *jcr, run_ctx &rc)
          add_prompt(ua, _("Replace"));       /* 10 */
          add_prompt(ua, _("JobId"));         /* 11 */
       }
-      if (jcr->getJobType() == JT_BACKUP || jcr->getJobType() == JT_RESTORE) {
-         add_prompt(ua, _("Plugin Options")); /* 12 */
+      if (jcr->getJobType() == JT_BACKUP || jcr->getJobType() == JT_RESTORE || jcr->is_JobType(JT_VERIFY)) {
+         add_prompt(ua, _("Plugin Options")); /* R:12 V:9  B:9*/
       }
       switch (do_prompt(ua, "", _("Select parameter to modify"), NULL, 0)) {
       case 0:
@@ -1379,19 +1381,33 @@ int modify_job_parameters(UAContext *ua, JCR *jcr, run_ctx &rc)
          jcr->where = bstrdup(ua->cmd);
          goto try_again;
       case 9:
-         /* File relocation */
-         select_where_regexp(ua, jcr);
-         goto try_again;
-      case 10:
-         /* Replace */
-         start_prompt(ua, _("Replace:\n"));
-         for (i=0; ReplaceOptions[i].name; i++) {
-            add_prompt(ua, ReplaceOptions[i].name);
+         if (jcr->getJobType() == JT_RESTORE) {
+            /* File relocation */
+            select_where_regexp(ua, jcr);
+            goto try_again;
          }
-         opt = do_prompt(ua, "", _("Select replace option"), NULL, 0);
-         if (opt >=  0) {
-            rc.replace = ReplaceOptions[opt].name;
-            jcr->replace = ReplaceOptions[opt].token;
+      case 10:
+         if (jcr->getJobType() == JT_RESTORE) {
+            /* Replace */
+            start_prompt(ua, _("Replace:\n"));
+            for (i=0; ReplaceOptions[i].name; i++) {
+               add_prompt(ua, ReplaceOptions[i].name);
+            }
+            opt = do_prompt(ua, "", _("Select replace option"), NULL, 0);
+            if (opt >=  0) {
+               rc.replace = ReplaceOptions[opt].name;
+               jcr->replace = ReplaceOptions[opt].token;
+            }
+         } else {
+            /* Plugin Options for verify data */
+            if (!get_cmd(ua, _("Please enter the Plugin Options string: "))) {
+               break;
+            }
+            if (jcr->plugin_options) {
+               free(jcr->plugin_options);
+               jcr->plugin_options = NULL;
+            }
+            jcr->plugin_options = bstrdup(ua->cmd);
          }
          goto try_again;
       case 11:
@@ -1403,22 +1419,8 @@ int modify_job_parameters(UAContext *ua, JCR *jcr, run_ctx &rc)
          }
          goto try_again;
       case 12:
-
          if (jcr->getJobType() == JT_RESTORE) {
             plugin_config(ua, jcr, rc);
-
-         } else {
-            //generate_plugin_event(jcr, bEventJobConfig, &rc);
-
-            /* Plugin Options */
-            if (!get_cmd(ua, _("Please Plugin Options string: "))) {
-               break;
-            }
-            if (jcr->plugin_options) {
-               free(jcr->plugin_options);
-               jcr->plugin_options = NULL;
-            }
-            jcr->plugin_options = bstrdup(ua->cmd);
          }
          goto try_again;
       case -1:                        /* error or cancel */
@@ -1558,9 +1560,7 @@ static bool set_run_context_in_jcr(UAContext *ua, JCR *jcr, run_ctx &rc)
    }
 
    if (rc.plugin_options) {
-      if (jcr->plugin_options) {
-         free(jcr->plugin_options);
-      }
+      bfree_and_null(jcr->plugin_options);
       jcr->plugin_options = bstrdup(rc.plugin_options);
       rc.plugin_options = NULL;
    }
@@ -2017,16 +2017,17 @@ static bool display_job_parameters(UAContext *ua, JCR *jcr, JOB *job, const char
             ua->signal(BNET_RUN_CMD);
             ua->send_msg("Type: Verify\n"
                         "Title: Run Verify Job\n"
-                        "JobName:     %s\n"
-                        "Level:       %s\n"
-                        "Client:      %s\n"
-                        "FileSet:     %s\n"
-                        "Pool:        %s (From %s)\n"
-                        "Storage:     %s (From %s)\n"
-                        "Verify Job:  %s\n"
-                        "Verify List: %s\n"
-                        "When:        %s\n"
-                        "Priority:    %d\n",
+                        "JobName:        %s\n"
+                        "Level:          %s\n"
+                        "Client:         %s\n"
+                        "FileSet:        %s\n"
+                        "Pool:           %s (From %s)\n"
+                        "Storage:        %s (From %s)\n"
+                        "Verify Job:     %s\n"
+                        "Verify List:    %s\n"
+                        "When:           %s\n"
+                        "Priority:       %d\n"
+                        "Plugin Options: %s\n",
               job->name(),
               level_to_str(edl, sizeof(edl), jcr->getJobLevel()),
               jcr->client->name(),
@@ -2036,19 +2037,21 @@ static bool display_job_parameters(UAContext *ua, JCR *jcr, JOB *job, const char
               Name,
               verify_list,
               bstrutime(dt, sizeof(dt), jcr->sched_time),
-              jcr->JobPriority);
+              jcr->JobPriority,
+              NPRT(jcr->plugin_options));
          } else {
             ua->send_msg(_("Run Verify Job\n"
-                        "JobName:     %s\n"
-                        "Level:       %s\n"
-                        "Client:      %s\n"
-                        "FileSet:     %s\n"
-                        "Pool:        %s (From %s)\n"
-                        "Storage:     %s (From %s)\n"
-                        "Verify Job:  %s\n"
-                        "Verify List: %s\n"
-                        "When:        %s\n"
-                        "Priority:    %d\n"),
+                        "JobName:        %s\n"
+                        "Level:          %s\n"
+                        "Client:         %s\n"
+                        "FileSet:        %s\n"
+                        "Pool:           %s (From %s)\n"
+                        "Storage:        %s (From %s)\n"
+                        "Verify Job:     %s\n"
+                        "Verify List:    %s\n"
+                        "When:           %s\n"
+                        "Priority:       %d\n"
+                        "Plugin Options: %s\n"),
               job->name(),
               level_to_str(edl, sizeof(edl), jcr->getJobLevel()),
               jcr->client->name(),
@@ -2058,7 +2061,8 @@ static bool display_job_parameters(UAContext *ua, JCR *jcr, JOB *job, const char
               Name,
               verify_list,
               bstrutime(dt, sizeof(dt), jcr->sched_time),
-              jcr->JobPriority);
+              jcr->JobPriority,
+              NPRT(jcr->plugin_options));
          }
       }
       break;
@@ -2559,15 +2563,13 @@ static bool scan_run_command_line_arguments(UAContext *ua, run_ctx &rc)
                kw_ok = true;
                break;
             case 25: /* pluginoptions */
-               ua->send_msg(_("Plugin Options not yet implemented.\n"));
-               return false;
                if (rc.plugin_options) {
                   ua->send_msg(_("Plugin Options specified twice.\n"));
                   return false;
                }
                rc.plugin_options = ua->argv[i];
                if (!acl_access_ok(ua, PluginOptions_ACL, rc.plugin_options)) {
-                  ua->send_msg(_("No authoriztion for \"PluginOptions\" specification.\n"));
+                  ua->send_msg(_("No authorization for \"PluginOptions\" specification.\n"));
                   return false;
                }
                kw_ok = true;
@@ -2715,7 +2717,6 @@ static bool scan_run_command_line_arguments(UAContext *ua, run_ctx &rc)
       return false;
    }
 
-
    if (!get_client(ua, rc)) {
       return false;
    }
@@ -2734,6 +2735,10 @@ static bool scan_run_command_line_arguments(UAContext *ua, run_ctx &rc)
       rc.verify_job = rc.job->verify_job;
    }
 
+   if (rc.job->JobType == JT_VERIFY && !rc.plugin_options && rc.job->PluginOptions) {
+      rc.plugin_options = rc.job->PluginOptions;
+   }
+
    if (rc.previous_job_name) {
       rc.previous_job = GetJobResWithName(rc.previous_job_name);
       if (!rc.previous_job) {
