@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -70,6 +70,7 @@ bool do_vbackup_init(JCR *jcr)
    Dmsg2(dbglevel, "Read pool=%s (From %s)\n", jcr->rpool->name(), jcr->rpool_source);
 
    jcr->start_time = time(NULL);
+   jcr->jr.isVirtualFull = 1;
    jcr->jr.StartTime = jcr->start_time;
    jcr->jr.JobLevel = L_FULL;      /* we want this to appear as a Full backup */
    if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
@@ -85,6 +86,25 @@ bool do_vbackup_init(JCR *jcr)
    return true;
 }
 
+/* TODO: Workaround waiting to get the FileIndex in the Object table. With this trick, a bscan will
+ * not re-create the objects in the catalog, So the object selection during the restore process
+ * is important.
+ */
+static bool copy_object_list(JCR *jcr, const char *jobids, uint32_t JobId)
+{
+   if (jcr->is_canceled()) {
+      return false;
+   }
+   db_lock(jcr->db_batch);
+   Mmsg(jcr->db_batch->cmd, copy_object[db_get_type_index(jcr->db_batch)], JobId, jobids, jobids);
+   if (!db_sql_query(jcr->db_batch, jcr->db_batch->cmd, NULL, NULL)) {
+      db_unlock(jcr->db_batch);
+      return false;
+   }
+   db_unlock(jcr->db_batch);
+   return true;
+}
+
 /*
  * Do a virtual backup, which consolidates all previous backups into
  *  a sort of synthetic Full.
@@ -101,7 +121,8 @@ bool do_vbackup(JCR *jcr)
    sellist     sel;
    db_list_ctx jobids;
    UAContext *ua;
-
+   bootstrap_info info;
+   POOL_MEM buf;
    Dmsg2(100, "rstorage=%p wstorage=%p\n", jcr->store_mngr->get_rstore_list(), jcr->store_mngr->get_wstore_list());
    Dmsg2(100, "Read store=%s, write store=%s\n",
       ((STORE *)jcr->store_mngr->get_rstore_list()->first())->name(),
@@ -179,8 +200,7 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
             return false;
          }
 
-         Jmsg(jcr, M_INFO, 0, _("Using user supplied JobIds=%s\n"),
-              jobids.list);
+         jmsg_large_jobid_list(jcr, _("Using user supplied JobIds=%s\n"), jobids.list);
 
          /* Check status */
          Mmsg(query,
@@ -241,7 +261,7 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
 
       } else {
          Jmsg(jcr, M_WARNING, 0, _("BackupsToKeep is set to %u, "
-                                   "and only %d Incremental/Differential jobs exits. "
+                                   "and/or only %d jobs exits. "
                                    "Consolidation not needed.\n"),
                                     jcr->job->BackupsToKeep, jobids.count);
          return false;
@@ -261,7 +281,8 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
    /* Full by default, or might be Incr/Diff when jobid= is used */
    jcr->jr.JobLevel = level_computed;
 
-   Jmsg(jcr, M_INFO, 0, "Consolidating JobIds=%s\n", jobids.list);
+   /* Display the JobId selection and adjust the output for Jmsg */
+   jmsg_large_jobid_list(jcr, _("Consolidating JobIds=%s\n"), jobids.list);
 
    /*
     * Now we find the last job that ran and store it's info in
@@ -284,9 +305,29 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
       return false;
    }
 
-   if (!create_bootstrap_file(jcr, jobids.list)) {
-      Jmsg(jcr, M_FATAL, 0, _("Could not get or create the FileSet record.\n"));
-      return false;
+   {
+      /* check if the BSR have volume cycle and return a FATAL error if so
+       * in the future we must modify VF to be able to handle such situation
+       */
+      if (!create_bootstrap_file(jcr, jobids.list)) {
+         Jmsg(jcr, M_FATAL, 0, _("Could not get or create the FileSet record.\n"));
+         return false;
+      }
+
+      /* Open the bootstrap file */
+      if (!open_bootstrap_file(jcr, info)) {
+         return false;
+      }
+
+      if (split_bsr_loop(jcr, info)) { /* create the split list to break volume cycle */
+         Jmsg(jcr, M_FATAL, 0, _("Found a volume cycle in the bootstrap, Virtual Full is not possible on this Job\n"));
+      }
+
+      close_bootstrap_file(info);
+
+      if (jcr->is_canceled()) {
+         return false;
+      }
    }
 
    /*
@@ -301,9 +342,15 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
     * Start conversation with Storage daemon
     */
    if (!connect_to_storage_daemon(jcr, 10, SDConnectTimeout, 1)) {
+      Jmsg(jcr, M_FATAL, 0, "%s", jcr->errmsg);
       return false;
    }
    sd = jcr->store_bsock;
+   build_connecting_info_log(_("Storage"), jcr->store_mngr->get_wstore()->name(),
+                             get_storage_address(jcr->client, jcr->store_mngr->get_wstore()),
+                             jcr->store_mngr->get_wstore()->SDport,
+                             sd->tls ? true : false, buf.addr());
+   Jmsg(jcr, M_INFO, 0, "%s", buf.c_str());
 
    /*
     * Now start a job with the Storage daemon
@@ -360,12 +407,31 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
    /* Note, the SD stores in jcr->JobFiles/ReadBytes/JobBytes/JobErrors */
    wait_for_storage_daemon_termination(jcr);
    jcr->setJobStatus(jcr->SDJobStatus);
+
+   /* At this point, the batch SQL link should be open */
+   if (!db_open_batch_connection(jcr, jcr->db)) {
+      Jmsg0(jcr, M_FATAL, 0, "Can't get batch sql connection");
+      return false;
+   }
+
    if (!flush_file_records(jcr)) {     /* cached attribute + batch insert */
       Jmsg(jcr, M_ERROR, 0,
            _("Unable to flush file records!\n"));
       return false;
    }
 
+   /* Waiting to copy objects with the BSR like RestoreObjects, we use SQL */
+   if (!copy_object_list(jcr, jobids.list, jcr->JobId))
+   {
+      Jmsg(jcr, M_ERROR, 0, _("Unable to copy objects ERR=%s\n"), jcr->db_batch->errmsg);
+   }
+
+   /* For some plugins, we need to maintain the PriorJob and PriorJobId for the new job */
+   if (!db_get_prior_job(jcr, jcr->db_batch, jobids.list, &jcr->jr)) {
+      Jmsg(jcr, M_ERROR, 0, _("Unable to find or set the PriorJob information to the new Job record ERR=%s\n"),
+                              jcr->db_batch->errmsg);
+   }
+
    if (jcr->JobStatus != JS_Terminated) {
       return false;
    }
@@ -373,7 +439,8 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
       ua = new_ua_context(jcr);
       purge_jobs_from_catalog(ua, jobids.list);
       free_ua_context(ua);
-      Jmsg(jcr, M_INFO, 0, _("Deleted consolidated JobIds=%s\n"), jobids.list);
+
+      jmsg_large_jobid_list(jcr, _("Delete consolidated JobIds=%s\n"), jobids.list);
    }
 
    vbackup_cleanup(jcr, jcr->JobStatus);
@@ -387,21 +454,28 @@ _("This Job is not an Accurate backup so is not equivalent to a Full backup.\n")
 void vbackup_cleanup(JCR *jcr, int TermCode)
 {
    char sdt[50], edt[50], schedt[50];
-   char ec1[30], ec3[30], ec4[30], compress[50];
+   char ec1[30], ec3[30], ec4[30];
    char ec7[30], ec8[30], elapsed[50];
    char term_code[100], sd_term_msg[100];
    const char *term_msg;
    int msg_type = M_INFO;
    MEDIA_DBR mr;
    CLIENT_DBR cr;
-   double kbps, compression;
-   utime_t RunTime;
    POOL_MEM query(PM_MESSAGE);
    STORE *wstore = jcr->store_mngr->get_wstore();
 
    Dmsg2(100, "Enter vbackup_cleanup %d %c\n", TermCode, TermCode);
    memset(&cr, 0, sizeof(cr));
 
+   /* Job needs to be marked as terminated before running the after runscript */
+   jcr->setJobStatus(TermCode);
+
+   run_scripts(jcr, jcr->job->RunScripts, "AtJobCompletion");
+
+   /* Runscript could have changed JobStatus,
+    * now check if it should be changed in the report or not */
+   TermCode = compareJobStatus(TermCode, jcr->JobStatus);
+
    jcr->jr.JobLevel = L_FULL;   /* we want this to appear as a Full backup */
    jcr->JobFiles = jcr->SDJobFiles;
    jcr->JobBytes = jcr->SDJobBytes;
@@ -428,13 +502,6 @@ void vbackup_cleanup(JCR *jcr, int TermCode)
          db_strerror(jcr->db));
    }
 
-   bstrncpy(mr.VolumeName, jcr->VolumeName, sizeof(mr.VolumeName));
-   if (!db_get_media_record(jcr, jcr->db, &mr)) {
-      Jmsg(jcr, M_WARNING, 0, _("Error getting Media record for Volume \"%s\": ERR=%s"),
-         mr.VolumeName, db_strerror(jcr->db));
-      jcr->setJobStatus(JS_ErrorTerminated);
-   }
-
    update_bootstrap_file(jcr);
 
    switch (jcr->JobStatus) {
@@ -466,12 +533,8 @@ void vbackup_cleanup(JCR *jcr, int TermCode)
    bstrftimes(schedt, sizeof(schedt), jcr->jr.SchedTime);
    bstrftimes(sdt, sizeof(sdt), jcr->jr.StartTime);
    bstrftimes(edt, sizeof(edt), jcr->jr.EndTime);
-   RunTime = jcr->jr.EndTime - jcr->jr.StartTime;
-   if (RunTime <= 0) {
-      RunTime = 1;
-   }
-   kbps = ((double)jcr->jr.JobBytes) / (1000.0 * (double)RunTime);
-   if (!db_get_job_volume_names(jcr, jcr->db, jcr->jr.JobId, &jcr->VolumeName)) {
+   if (!db_get_job_volume_names(jcr, jcr->db, jcr->jr.JobId, &jcr->VolumeName,
+         mr.VolumeName, sizeof(mr.VolumeName))) {
       /*
        * Note, if the job has erred, most likely it did not write any
        *  tape, so suppress this "error" message since in that case
@@ -482,18 +545,14 @@ void vbackup_cleanup(JCR *jcr, int TermCode)
          Jmsg(jcr, M_ERROR, 0, "%s", db_strerror(jcr->db));
       }
       jcr->VolumeName[0] = 0;         /* none */
-   }
-
-   if (jcr->ReadBytes == 0) {
-      bstrncpy(compress, "None", sizeof(compress));
    } else {
-      compression = (double)100 - 100.0 * ((double)jcr->JobBytes / (double)jcr->ReadBytes);
-      if (compression < 0.5) {
-         bstrncpy(compress, "None", sizeof(compress));
-      } else {
-         bsnprintf(compress, sizeof(compress), "%.1f %%", compression);
+      if (!db_get_media_record(jcr, jcr->db, &mr)) {
+         Jmsg(jcr, M_WARNING, 0, _("Error getting Media record for Volume \"%s\": ERR=%s"),
+            mr.VolumeName, db_strerror(jcr->db));
+         jcr->setJobStatus(JS_ErrorTerminated);
       }
    }
+
    jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
 
    Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
@@ -533,12 +592,12 @@ void vbackup_cleanup(JCR *jcr, int TermCode)
         schedt,
         sdt,
         edt,
-        edit_utime(RunTime, elapsed, sizeof(elapsed)),
+        edit_utime(jcr->jr.RunTime, elapsed, sizeof(elapsed)),
         jcr->JobPriority,
         edit_uint64_with_commas(jcr->jr.JobFiles, ec1),
         edit_uint64_with_commas(jcr->jr.JobBytes, ec3),
         edit_uint64_with_suffix(jcr->jr.JobBytes, ec4),
-        kbps,
+        jcr->jr.Rate,
         jcr->VolumeName,
         jcr->VolSessionId,
         jcr->VolSessionTime,
@@ -570,6 +629,7 @@ int insert_bootstrap_handler(void *ctx, int num_fields, char **row)
 
    JobId = str_to_int64(row[3]);
    FileIndex = str_to_int64(row[2]);
+   Dmsg2(10, "insert bootstrap(%ld, %d)\n", JobId, FileIndex);
    add_findex(bsr_list, JobId, FileIndex);
    return 0;
 }
@@ -588,15 +648,15 @@ static bool create_bootstrap_file(JCR *jcr, char *jobids)
 
 #define new_get_file_list
 #ifdef new_get_file_list
-   if (!db_open_batch_connexion(jcr, jcr->db)) {
-      Jmsg0(jcr, M_FATAL, 0, "Can't get batch sql connexion");
+   if (!db_open_batch_connection(jcr, jcr->db)) {
+      Jmsg0(jcr, M_FATAL, 0, "Can't get batch sql connection");
       return false;
    }
    /* If the new Job is a Full, we don't need to keep deleted records,
     * however, if it's an incremental/differential, we should not restore
     * the files from the Full, so we keep the deleted records
     */
-   int opt = DBL_USE_DELTA;
+   int opt = DBL_USE_DELTA | DBL_USE_OBJ;
    if (jcr->jr.JobLevel != 'F') {
       opt |= DBL_ALL_FILES;
    }
@@ -635,5 +695,6 @@ static bool create_bootstrap_file(JCR *jcr, char *jobids)
         jcr->ExpectedFiles);
    free_ua_context(ua);
    free_bsr(rx.bsr_list);
+   dir_close_batch_connection(jcr);
    return jcr->ExpectedFiles==0?false:true;
 }
