@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -62,19 +62,13 @@ const bool have_xattr = false;
 /* Data received from Storage Daemon */
 static char rec_header[] = "rechdr %ld %ld %ld %ld %ld";
 
-/* Forward referenced functions */
-#if   defined(HAVE_LIBZ)
+#ifdef HAVE_LIBZ
 static const char *zlib_strerror(int stat);
-const bool have_libz = true;
-#else
-const bool have_libz = false;
-#endif
-#ifdef HAVE_LZO
-const bool have_lzo = true;
-#else
-const bool have_lzo = false;
 #endif
 
+/* Get have_lzo, have_zstd, ... */
+#include "compress.h"
+
 static void deallocate_cipher(r_ctx &rctx);
 static void deallocate_fork_cipher(r_ctx &rctx);
 static bool verify_signature(r_ctx &rctx);
@@ -263,6 +257,7 @@ static inline bool pop_delayed_data_streams(r_ctx &rctx)
 {
    RESTORE_DATA_STREAM *rds;
    JCR *jcr = rctx.jcr;
+   char buf[80]; /* for asciidump() */
 
    /*
     * See if there is anything todo.
@@ -331,9 +326,10 @@ static inline bool pop_delayed_data_streams(r_ctx &rctx)
          }
          break;
       default:
-         Jmsg(jcr, M_WARNING, 0, _("Unknown stream=%d ignored. This shouldn't happen!\n"),
-              rds->stream);
-         Dmsg2(0, "Unknown stream=%d data=%s\n", rds->stream, rds->content);
+         Jmsg(jcr, M_WARNING, 0, _("Unexpected stream \"%s\" ignored. This shouldn't happen!\n"),
+               stream_to_ascii(rds->stream));
+         Dmsg2(50, "Unexpected stream=%d data=%s...\n", rds->stream,
+               asciidump(rds->content, rds->content_length, buf, sizeof(buf)));
          break;
       }
       if (rds->content) {
@@ -366,6 +362,7 @@ void do_restore(JCR *jcr)
    r_ctx rctx;
    ATTR *attr;
    int bget_ret = 0;
+   char buf[80];                       /* for asciidump() */
    /* ***FIXME*** make configurable */
    crypto_digest_t signing_algorithm = have_sha2 ?
                                        CRYPTO_DIGEST_SHA256 : CRYPTO_DIGEST_SHA1;
@@ -400,7 +397,7 @@ void do_restore(JCR *jcr)
    jcr->buf_size = sd->msglen;
 
    /* use the same buffer size to decompress both gzip and lzo */
-   if (have_libz || have_lzo) {
+   if (have_libz || have_lzo || have_zstd) {
       uint32_t compress_buf_size = jcr->buf_size + 12 + ((jcr->buf_size+999) / 1000) + 100;
       jcr->compress_buf = get_memory(compress_buf_size);
       jcr->compress_buf_size = compress_buf_size;
@@ -411,11 +408,16 @@ void do_restore(JCR *jcr)
    fdmsg->start_read_sock();
    bmessage *bmsg = fdmsg->new_msg(); /* get a message, to exchange with fdmsg */
 
+#ifdef HAVE_ZSTD
+      ZSTD_DCtx *pZSTD = ZSTD_createDCtx();
+      jcr->ZSTD_decompress_workset = pZSTD;
+#endif
+
 #ifdef HAVE_LZO
-   if (lzo_init() != LZO_E_OK) {
-      Jmsg(jcr, M_FATAL, 0, _("LZO init failed\n"));
-      goto get_out;
-   }
+      if (lzo_init() != LZO_E_OK) {
+         Jmsg(jcr, M_FATAL, 0, _("LZO init failed\n"));
+         goto get_out;
+      }
 #endif
 
    if (have_crypto) {
@@ -466,15 +468,7 @@ void do_restore(JCR *jcr)
 
    Dsm_check(200);
    while ((bget_ret = fdmsg->bget_msg(&bmsg)) >= 0 && !job_canceled(jcr)) {
-      time_t now = time(NULL);
-      if (jcr->last_stat_time == 0) {
-         jcr->last_stat_time = now;
-         jcr->stat_interval = 30;  /* Default 30 seconds */
-      } else if (now >= jcr->last_stat_time + jcr->stat_interval) {
-         jcr->dir_bsock->fsend("Progress JobId=%ld files=%ld bytes=%lld bps=%ld\n",
-            jcr->JobId, jcr->JobFiles, jcr->JobBytes, jcr->LastRate);
-         jcr->last_stat_time = now;
-      }
+      jcr->sendProgressStatus();
 
       /* Remember previous stream type */
       rctx.prev_stream = rctx.stream;
@@ -1042,6 +1036,9 @@ void do_restore(JCR *jcr)
       case STREAM_SHA1_DIGEST:
       case STREAM_SHA256_DIGEST:
       case STREAM_SHA512_DIGEST:
+      case STREAM_XXHASH64_DIGEST:
+      case STREAM_XXH3_64_DIGEST:
+      case STREAM_XXH3_128_DIGEST:
          break;
 
       case STREAM_PROGRAM_NAMES:
@@ -1060,6 +1057,7 @@ void do_restore(JCR *jcr)
          plugin_name_stream(jcr, bmsg->rbuf);
          break;
 
+      case STREAM_FILEEVENT:
       case STREAM_RESTORE_OBJECT:
          break;                    /* these are sent by Director */
 
@@ -1109,9 +1107,10 @@ void do_restore(JCR *jcr)
          if (!close_previous_stream(rctx)) {
             goto get_out;
          }
-         Jmsg(jcr, M_WARNING, 0, _("Unknown stream=%d ignored. This shouldn't happen!\n"),
-              rctx.stream);
-         Dmsg2(0, "Unknown stream=%d data=%s\n", rctx.stream, bmsg->rbuf);
+         Jmsg(jcr, M_WARNING, 0, _("Unexpected stream=\"%s\" ignored. This shouldn't happen!\n"),
+               stream_to_ascii(rctx.stream));
+         Dmsg2(50, "Unexpected stream=%d data=%s\n", rctx.stream,
+               asciidump(bmsg->rbuf, bmsg->rbuflen, buf, sizeof(buf)));
          break;
       } /* end switch(stream) */
 
@@ -1221,6 +1220,9 @@ ok_out:
       jcr->compress_buf = NULL;
       jcr->compress_buf_size = 0;
    }
+#ifdef HAVE_ZSTD
+   ZSTD_freeDCtx((ZSTD_DCtx_s*)jcr->ZSTD_decompress_workset);
+#endif
 
 #ifdef HAVE_ACL
    if (jcr->bacl) {
@@ -1312,7 +1314,7 @@ bool sparse_data(JCR *jcr, BFILE *bfd, uint64_t *addr, char **data, uint32_t *le
 
 bool decompress_data(JCR *jcr, int32_t stream, char **data, uint32_t *length)
 {
-#if defined(HAVE_LZO) || defined(HAVE_LIBZ)
+#if defined(HAVE_LZO) || defined(HAVE_LIBZ) || defined(HAVE_ZSTD)
    char ec1[50];                   /* Buffer printing huge values */
 #endif
 
@@ -1327,6 +1329,9 @@ bool decompress_data(JCR *jcr, int32_t stream, char **data, uint32_t *length)
       const unsigned char *cbuf;
       int r, real_compress_len;
 #endif
+#ifdef HAVE_ZSTD
+      unsigned long long rSize;
+#endif
 
       /* read compress header */
       unser_declare;
@@ -1376,6 +1381,32 @@ bool decompress_data(JCR *jcr, int32_t stream, char **data, uint32_t *length)
             *length = compress_len;
             Dmsg2(200, "Write uncompressed %d bytes, total before write=%s\n", compress_len, edit_uint64(jcr->JobBytes, ec1));
             return true;
+#endif
+#ifdef HAVE_ZSTD
+         case COMPRESS_ZSTD:
+            compress_len = jcr->compress_buf_size;
+            cbuf = (const unsigned char*)*data + sizeof(comp_stream_header);
+            real_compress_len = *length - sizeof(comp_stream_header);
+            Dmsg2(200, "Comp_len=%d msglen=%d\n", compress_len, *length);
+            rSize = ZSTD_getFrameContentSize(cbuf, real_compress_len);
+            if (rSize == ZSTD_CONTENTSIZE_ERROR || rSize == ZSTD_CONTENTSIZE_UNKNOWN) {
+               Qmsg(jcr, M_ERROR, 0, _("ZSTD uncompression error on file %s. Invalid content size. ERR=%d\n"),
+                    jcr->last_fname, rSize);
+               return false;
+            }
+            jcr->compress_buf = check_pool_memory_size(jcr->compress_buf, rSize);
+            rSize = ZSTD_decompressDCtx((ZSTD_DCtx*)jcr->ZSTD_decompress_workset,
+                                        (unsigned char *)jcr->compress_buf, compress_len, cbuf, real_compress_len);
+
+            if (rSize == ZSTD_CONTENTSIZE_ERROR || rSize == ZSTD_CONTENTSIZE_UNKNOWN) {
+               Qmsg(jcr, M_ERROR, 0, _("ZSTD uncompression error on file %s. Decompress error. ERR=%d\n"),
+                    jcr->last_fname, rSize);
+               return false;
+            }
+            *data = jcr->compress_buf;
+            *length = compress_len = rSize;
+            Dmsg2(200, "Write uncompressed %d bytes, total before write=%s\n", compress_len, edit_uint64(jcr->JobBytes, ec1));
+            return true;
 #endif
          default:
             Qmsg(jcr, M_ERROR, 0, _("Compression algorithm 0x%x found, but not supported!\n"), comp_magic);
@@ -1696,7 +1727,6 @@ bool flush_cipher(r_ctx &rctx, BFILE *bfd, uint64_t *addr, int flags, int32_t st
    char ec1[50];                      /* Buffer printing huge values */
    bool second_pass = false;
 
-again:
    /* Write out the remaining block and free the cipher context */
    cipher_ctx->buf = check_pool_memory_size(cipher_ctx->buf,
                         cipher_ctx->buf_len + cipher_ctx->block_size);
@@ -1716,6 +1746,7 @@ again:
 
    cipher_ctx->buf_len += decrypted_len;
 
+again:
    unser_crypto_packet_len(cipher_ctx);
    Dmsg1(500, "Crypto unser block size=%d\n", cipher_ctx->packet_len - CRYPTO_LEN_SIZE);
    wsize = cipher_ctx->packet_len - CRYPTO_LEN_SIZE;
