@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -154,6 +154,7 @@ static int verify_file(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
       Jmsg(jcr, M_SKIPPED, 1, _("     File system change prohibited. Directory skipped: %s\n"), ff_pkt->fname);
       return 1;
    case FT_PLUGIN_CONFIG:
+   case FT_SECURITY_OBJECT:
    case FT_RESTORE_FIRST:
       return 1;                       /* silently skip */
    case FT_NOOPEN: {
@@ -237,6 +238,18 @@ static int verify_file(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
       } else if (ff_pkt->flags & FO_SHA512) {
          digest = crypto_digest_new(jcr, CRYPTO_DIGEST_SHA512);
          digest_stream = STREAM_SHA512_DIGEST;
+
+      } else if (ff_pkt->flags & FO_XXHASH64) {
+         digest = crypto_digest_new(jcr, CRYPTO_DIGEST_XXHASH64);
+         digest_stream = STREAM_XXHASH64_DIGEST;
+
+      } else if (ff_pkt->flags & FO_XXH3_64) {
+         digest = crypto_digest_new(jcr, CRYPTO_DIGEST_XXH3_64);
+         digest_stream = STREAM_XXH3_64_DIGEST;
+
+      } else if (ff_pkt->flags & FO_XXH3_128) {
+         digest = crypto_digest_new(jcr, CRYPTO_DIGEST_XXH3_128);
+         digest_stream = STREAM_XXH3_128_DIGEST;
       }
 
       /* Did digest initialization fail? */
@@ -245,7 +258,7 @@ static int verify_file(JCR *jcr, FF_PKT *ff_pkt, bool top_level)
               stream_to_ascii(digest_stream));
       }
 
-      /* compute MD5 or SHA1 hash */
+      /* compute MD5 or SHA??? or XXH??? hash */
       if (digest) {
          char md[CRYPTO_DIGEST_MAX_SIZE];
          uint32_t size;
