@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2020 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -11,7 +11,7 @@
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   This notice must be preserved when any source code is 
+   This notice must be preserved when any source code is
    conveyed and/or propagated.
 
    Bacula(R) is a registered trademark of Kern Sibbald.
@@ -104,6 +104,8 @@ static void s_err(const char *file, int line, LEX *lc, const char *msg, ...)
    } else {
       e_msg(file, line, lc->err_type, 0, _("Config error: %s\n"), buf);
    }
+
+   lc->last_result = -1;
 }
 
 void lex_set_default_error_handler(LEX *lf)
@@ -240,7 +242,7 @@ LEX *lex_open_file(LEX *lf, const char *filename, LEX_ERROR_HANDLER *scan_error)
    char *fname = bstrdup(filename);
 
    if (fname[0] == '|') {
-      if ((bpipe = open_bpipe(fname+1, 0, "rb")) == NULL) {
+      if ((bpipe = open_bpipe(fname+1, 0, "rEb")) == NULL) {
          free(fname);
          return NULL;
       }
