@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -80,6 +80,12 @@
 #endif
 #endif
 
+#ifdef DEVELOPER
+#define CHECK_INTERVAL 30
+#else
+#define CHECK_INTERVAL 1200
+#endif
+
 /*
   Inspired from
   http://www.cs.berkeley.edu/~kamil/teaching/sp03/041403.pdf
@@ -762,7 +768,7 @@ void *check_deadlock(void *)
 
       gettimeofday(&tv, &tz);
       timeout.tv_nsec = 0;
-      timeout.tv_sec = tv.tv_sec + 30;
+      timeout.tv_sec = tv.tv_sec + CHECK_INTERVAL;
 
       pthread_mutex_lock(&undertaker_mutex);
       pthread_cond_timedwait(&undertaker_cond, &undertaker_mutex, &timeout);
