@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -377,8 +377,7 @@ void garbage_collect_memory_pool()
       return;
    }
    now = time(NULL);
-   if (now >= last_garbage_collection + garbage_interval ||
-       sm_bytes > 500000) {
+   if (now >= last_garbage_collection + garbage_interval) {
       last_garbage_collection = now;
       V(mutex);
       garbage_collect_memory();
