@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -197,14 +197,16 @@ bool get_trace(void);
 void set_debug_flags(char *options);
 const char *get_basename(const char *pathname);
 bool is_message_type_set(JCR *jcr, int type);
-void set_trace_for_tools(FILE *new_trace_fd); // called by Bacula's tools only
+void set_trace_for_tools(int new_trace_fd); // called by Bacula's tools only
 void update_trace_file_location(bool safe);
 char *build_connecting_info_log(const char *daemon, const char *name, const char *addr, int port, bool tls, POOLMEM *&buf);
+int get_component_statuscode(const char *component);
 
 class BDB;                                              /* define forward reference */
 typedef bool (*sql_insert_log)(JCR *jcr, JobId_t jobid, utime_t mtime, char *msg);
 typedef bool (*sql_insert_event)(JCR *jcr, utime_t mtime, const char *line);
 
+extern DLL_IMP_EXP char component_code;
 extern DLL_IMP_EXP sql_insert_log  p_sql_log;
 extern DLL_IMP_EXP sql_insert_event p_sql_event;
 
