@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -174,6 +174,12 @@ void serial_string(uint8_t * * const ptr, const char * const str)
 // Dmsg2(000, "ser src=%s dest=%s\n", src, dest);
 }
 
+/* skip some bytes without writing anything */
+
+void serial_skip(uint8_t * * const ptr, int len)
+{
+    *ptr += len;
+}
 
 /*  unserial_int16  --  Unserialise a signed 16 bit integer.  */
 
