@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -202,9 +202,7 @@ static bRC freePlugin(bpContext *ctx)
    if (!p_ctx) {
       return bRC_Error;
    }
-   if (p_ctx->cmd) {
-      free(p_ctx->cmd);                  /* free any allocated command string */
-   }
+   bfree_and_null(p_ctx->cmd);
    free_and_null_pool_memory(p_ctx->restore_obj_buf);
    if (p_ctx->rop_writer) {
       restoreobj *rop;
