@@ -32,6 +32,7 @@ from baculak8s.plugins import k8sbackend
 from baculak8s.plugins.k8sbackend.baculabackup import BACULABACKUPPODNAME
 from baculak8s.plugins.k8sbackend.baculaannotations import annotated_namespaced_pods_data
 from baculak8s.plugins.k8sbackend.configmaps import *
+from baculak8s.plugins.k8sbackend.csi_snapshot import *
 from baculak8s.plugins.k8sbackend.daemonset import *
 from baculak8s.plugins.k8sbackend.deployment import *
 from baculak8s.plugins.k8sbackend.endpoints import *
@@ -51,13 +52,14 @@ from baculak8s.plugins.k8sbackend.service import *
 from baculak8s.plugins.k8sbackend.serviceaccounts import *
 from baculak8s.plugins.k8sbackend.statefulset import *
 from baculak8s.plugins.k8sbackend.storageclass import *
+from baculak8s.plugins.k8sbackend.volumesnapshotclass import get_snapshot_drivers_compatible
 from baculak8s.plugins.plugin import *
 from baculak8s.util.date_util import gmt_to_unix_timestamp
+from baculak8s.util.lambda_util import wait_until_resource_is_ready
 
 HTTP_NOT_FOUND = 404
 K8S_POD_CONTAINER_STATUS_ERROR = "Pod status error! Reason: {}, Message: {}"
-
-
+K8S_VOLUME_SNAPSHOT_STATUS_ERROR = "Volume status error! Reason: {}, Message: {}"
 class KubernetesPlugin(Plugin):
     """
         Plugin that communicates with Kubernetes API
@@ -79,7 +81,7 @@ class KubernetesPlugin(Plugin):
                 # pvcdata without value
                 _pvcdata = True
             else:
-                _pvcdata = [_pvcdata]
+                _pvcdata = _pvcdata.split(',')
 
         self.config = {
             'config_file': params.get("config", None),
@@ -123,6 +125,10 @@ class KubernetesPlugin(Plugin):
             K8SObjType.K8SOBJ_PVCDATA: {},
         }
         self._connected = False
+        self.pods_counter = 0
+        self.pvcs_counter = 0
+        self.pvcs_totalsize = 0
+        self.po_source_data = "<notset>"
 
     def connect(self):
         """
@@ -180,12 +186,12 @@ class KubernetesPlugin(Plugin):
         self.corev1api = client.CoreV1Api(api_client=self.clientAPI)
         self.appsv1api = client.AppsV1Api(api_client=self.clientAPI)
         self.storagev1api = client.StorageV1Api(api_client=self.clientAPI)
+        self.crd_api = client.CustomObjectsApi(api_client=self.clientAPI) # To manage csi snapshots
 
         logging.getLogger(requests.packages.urllib3.__package__).setLevel(logging.ERROR)
         logging.getLogger(client.rest.__package__).setLevel(logging.ERROR)
         urllib3.disable_warnings()
         logging.captureWarnings(True)
-
         response = self.__execute(lambda: self.coreapi.get_api_versions(), check_connection=False)
         if isinstance(response, dict) and "error" in response:
             logging.debug("ERROR response:{}".format(response))
@@ -204,6 +210,9 @@ class KubernetesPlugin(Plugin):
                 data = {}
             else:
                 data = {'response': response}
+                response = self.__execute(lambda: self.coreapi.get_api_versions(), check_connection=False)
+                if not (isinstance(response, dict) and "error" in response):
+                    self.po_source_data = response.server_address_by_client_cid_rs[0].server_address
             return data
 
     def disconnect(self):
@@ -328,13 +337,33 @@ class KubernetesPlugin(Plugin):
                                                                 self.config['labels']))
 
     def get_pods(self, namespace, estimate=False):
-        return self.__execute(lambda: pods_list_namespaced(self.corev1api, namespace, estimate, self.config['labels']))
+        # the __execute() wrapper intercept exceptions and return and error dict()
+        # instead of the expected response
+        response = self.__execute(lambda: pods_list_namespaced(self.corev1api, namespace, estimate, self.config['labels']))
+        if not isinstance(response, dict) or not "error" in response:
+            pods = response
+            nrpods = len(pods)
+            logging.debug("get_pods[{}]:pods:{}".format(namespace, nrpods))
+            self.pods_counter += nrpods
+            return pods
+        else:
+            return response # it is a dictionary with an error
 
     def get_pvcs(self, namespace, estimate=False):
-        pvcs = self.__execute(lambda: persistentvolumeclaims_list_namespaced(self.corev1api, namespace, estimate,
+        # the __execute() wrapper intercept exceptions and return and error dict()
+        # instead of the expected response
+        response = self.__execute(lambda: persistentvolumeclaims_list_namespaced(self.corev1api, namespace, estimate,
                                                                              self.config['labels']))
-        self.k8s['pvcs'] = pvcs
-        return pvcs
+        if isinstance(response, tuple) and len(response) == 2:
+            pvcs, totalsize = response
+            nrpvcs = len(pvcs)
+            self.pvcs_counter += nrpvcs
+            self.pvcs_totalsize += totalsize
+            self.k8s['pvcs'] = pvcs
+            logging.debug("get_pvcs[{}]:pvcs:{}".format(namespace, nrpvcs))
+            return pvcs
+        else:
+            return response # it should be a dictionary with an error
 
     def get_podtemplates(self, namespace, estimate=False):
         return self.__execute(lambda: podtemplates_list_namespaced(self.corev1api, namespace, estimate,
@@ -486,14 +515,14 @@ class KubernetesPlugin(Plugin):
         method_name = 'upload_' + str(K8SObjType.methoddict.get(file_info.objtype))
         method = getattr(self, method_name, None)
         if method is None:
-            return {'error': 'Invalid object type: %s' % file_info.objtype}
+            return {'error': 'Invalid object type: {}'.format(file_info.objtype)}
         if file_info.objcache is None:
-            curent_file = self.check_file(file_info)
-            if isinstance(curent_file, dict) and 'error' in curent_file:
+            current_file = self.check_file(file_info)
+            if isinstance(current_file, dict) and 'error' in current_file:
                 file_info.objcache = None
-                logging.error("check_file: {}".format(curent_file['error']))
+                logging.error("check_file: {}".format(current_file['error']))
             else:
-                file_info.objcache = curent_file
+                file_info.objcache = current_file
         return method(file_info, file_content)
 
     def restore_file(self, file_info, file_content_source=None):
@@ -504,6 +533,21 @@ class KubernetesPlugin(Plugin):
             return self.__restore_k8s_object(file_info, file_content_source)
 
     # TODO: export/move all checks into k8sbackend
+    def check_storage_compatibility_with_vsnapshot(self, storage_class_name):
+        logging.debug("Check Storage class compatibility.{}".format(storage_class_name))
+        storage_provisioner = get_provisioner(self.storagev1api, storage_class_name)
+        logging.debug("Provisioner {}".format(storage_provisioner))
+        logging.debug('Compatible Drivers: {} '.format(get_snapshot_drivers_compatible(self.crd_api)))
+        if storage_provisioner in get_snapshot_drivers_compatible(self.crd_api):
+            return True
+        return False
+
+    def check_pvc_compatiblity_with_vsnapshot(self, namespace, pvc_name):
+        pvc = self.get_pvcdata_namespaced(namespace, pvc_name)
+        logging.debug('Check Compatibility with Snapshots. Name: {}'.format(pvc_name))
+        logging.debug('PVC: {}'.format(pvc))
+        return self.check_storage_compatibility_with_vsnapshot(pvc.get('storage_class_name'))
+
     def _check_config_map(self, file_info):
         return self.__exec_check_object(
             lambda: self.corev1api.read_namespaced_config_map(k8sfile2objname(file_info.name), file_info.namespace))
@@ -593,7 +637,7 @@ class KubernetesPlugin(Plugin):
         method_name = '_check_' + str(K8SObjType.methoddict.get(file_info.objtype))
         method = getattr(self, method_name, None)
         if method is None:
-            return {'error': 'Invalid object type: %s' % file_info.objtype}
+            return {'error': 'Invalid object type: {}'.format(file_info.objtype)}
         return method(file_info)
 
     def __exec_check_object(self, action, check_connection=True):
@@ -750,12 +794,72 @@ class KubernetesPlugin(Plugin):
             return response
         return {}
 
+    def vsnapshot_isready(self, namespace, snapshot_name):
+        response = self._vsnapshot_status(namespace, snapshot_name)
+        if isinstance(response, dict) and "error" in response:
+            return response
+        status = response.get('status')
+        logging.debug("vsnapshot_isready:ReadyToUse:{}".format(status))
+        if status is None:
+            return False
+        return status.get('readyToUse')
+
+    def create_vsnapshot(self, namespace, pvc):
+        pvcname = pvc.get('name')
+        logging.info("Creating vsnapshot of pvc `{}`".format(pvc))
+        storage_provisioner =  get_provisioner(self.storagev1api, pvc.get('storage_class_name'))
+        snapshot = prepare_create_snapshot_body(self.crd_api, namespace, pvcname, self._params.get('jobid'), storage_provisioner)
+        logging.debug("Body to create vsnapshot:\n`{}`\n".format(snapshot))
+        response = self.__execute(lambda: self.crd_api.create_namespaced_custom_object(**snapshot, pretty=True))
+        if isinstance(response, dict) and "error" in response:
+            return response
+        snapshot_name = snapshot.get("body").get("metadata").get("name")
+        logging.debug('Request create snapshot `{}` sent correctly'.format(snapshot_name))
+        wait_until_resource_is_ready(
+            lambda: self.vsnapshot_isready(namespace, snapshot_name),
+            "Waiting create volume snapshot `{}` to be ready.".format(snapshot_name)
+        )
+        logging.info("Created successfully vsnapshot of pvc `{}`".format(pvcname))
+        return {
+            'name': snapshot_name,
+            'kind': snapshot.get('body').get('kind'),
+            'source': pvcname,
+            'namespace': namespace,
+            'volume_snapshot_class_name': snapshot.get('body').get('spec').get('volumeSnapshotClassName')
+        }
+
+    def create_pvc_from_vsnapshot(self, namespace, pvcdata):
+        new_pvc = prepare_pvc_from_vsnapshot_body(namespace, pvcdata, self._params.get('jobid'))
+        logging.debug("Body to create pvc from vsnapshot: `{}`".format(new_pvc))
+        response = self.create_pvc_clone(namespace, new_pvc)
+        if isinstance(response, dict) and 'error' in response:
+            return response
+        response = self.get_pvcdata_namespaced(namespace, new_pvc.get('metadata').get('name'))
+        if isinstance(response, dict) and 'error' in response:
+            return response
+        logging.debug('PVC from vsnapshot Response: {}'.format(response))
+        new_pvc_name = response.get('name')
+        wait_until_resource_is_ready(
+            lambda: self.pvc_status(namespace, new_pvc_name),
+            'Waiting create pvc from snapshot `{}` to be ready'.format(new_pvc_name)
+        )
+        return {
+            'name': new_pvc_name,
+            'node_name': response.get('node_name'),
+            'storage_class_name': response.get('storage_class_name'),
+            'capacity': response.get('capacity'),
+            'fi': pvcdata.get('fi')
+        }
+
     def backup_pod_status(self, namespace):
         return self.corev1api.read_namespaced_pod_status(name=BACULABACKUPPODNAME, namespace=namespace)
 
     def pvc_status(self, namespace, pvcname):
         return self.__execute(lambda: self.corev1api.read_namespaced_persistent_volume_claim_status(name=pvcname, namespace=namespace))
 
+    def _vsnapshot_status(self, namespace, snapshot_name):
+        return self.__execute(lambda: self.crd_api.get_namespaced_custom_object_status(**prepare_snapshot_action(namespace, snapshot_name)))
+
     def backup_pod_isready(self, namespace, seq=None, podname=BACULABACKUPPODNAME):
         pod = self.backup_pod_status(namespace)
         status = pod.status
@@ -789,12 +893,26 @@ class KubernetesPlugin(Plugin):
         return {}
 
     def remove_pvcclone(self, namespace, clonename):
-        logging.debug('remove_pvcclone')
+        logging.debug('remove_pvcclone `{}`'.format(clonename))
         response = self.__execute(lambda: self.corev1api.delete_namespaced_persistent_volume_claim(
             clonename, namespace, grace_period_seconds=0,
             propagation_policy='Foreground'))
+        logging.debug("Response: {}".format(response))
         if isinstance(response, dict) and "error" in response:
             return response
+        r1 = self.get_pvcdata_namespaced(namespace, clonename)
+        wait_until_resource_is_ready(lambda: 'error' in self.get_pvcdata_namespaced(namespace, clonename))
+        r2 = self.get_pvcdata_namespaced(namespace, clonename)
+        logging.debug('PVC removed `{}`'.format(clonename))
+        return {}
+
+    def remove_vsnapshot(self, namespace, vsnapshot_name):
+        logging.debug('remove_vsnapshot `{}`'.format(vsnapshot_name))
+        response = self.__execute(lambda: self.crd_api.delete_namespaced_custom_object(**prepare_snapshot_action(namespace, vsnapshot_name)))
+        logging.debug('Response remove_vsnapshot {}'.format(response))
+        if isinstance(response, dict) and "error" in response:
+            return response
+        logging.debug('Volume Snapshot removed `{}`'.format(vsnapshot_name))
         return {}
 
     def check_gone_backup_pod(self, namespace, force=False):
