@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -60,6 +60,7 @@ bool init_autochangers()
          if (!device->changer_command && changer->changer_command) {
             device->changer_command = bstrdup(changer->changer_command);
          }
+         /* ***BEEF*** */
          if (!device->lock_command && changer->lock_command) {
             device->lock_command = bstrdup(changer->lock_command);
          }
@@ -197,7 +198,7 @@ int autoload_device(DCR *dcr, bool writing, BSOCK *dir)
             new_vol_name, slot, drive);
 
          dcr->VolCatInfo.Slot = slot;    /* slot to be loaded */
-         changer = edit_device_codes(dcr, changer, dcr->device->changer_command, "load");
+         edit_device_codes(dcr, &changer, dcr->device->changer_command, "load");
          dev->close(dcr);
          Dmsg1(dbglvl, "Run program=%s\n", changer);
          status = run_program_full_output(changer, timeout, results.addr());
@@ -284,7 +285,7 @@ int get_autochanger_loaded_slot(DCR *dcr)
       Jmsg(jcr, M_INFO, 0, _("3301 Issuing autochanger \"loaded? drive %d\" command.\n"),
            drive);
    }
-   changer = edit_device_codes(dcr, changer, dcr->device->changer_command, "loaded");
+   edit_device_codes(dcr, &changer, dcr->device->changer_command, "loaded");
    Dmsg1(dbglvl, "Run program=%s\n", changer);
    status = run_program_full_output(changer, timeout, results.addr());
    Dmsg3(dbglvl, "run_prog: %s stat=%d result=%s", changer, status, results.c_str());
@@ -363,7 +364,6 @@ bool unload_autochanger(DCR *dcr, int loaded)
    DEVICE *dev = dcr->dev;
    JCR *jcr = dcr->jcr;
    const char *old_vol_name;
-   int slot;
    uint32_t timeout = dcr->device->max_changer_wait;
    bool ok = true;
 
@@ -398,6 +398,7 @@ bool unload_autochanger(DCR *dcr, int loaded)
    if (loaded > 0) {
       POOL_MEM results(PM_MESSAGE);
       POOLMEM *changer = get_pool_memory(PM_FNAME);
+      int slot;
       Jmsg(jcr, M_INFO, 0,
          _("3307 Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n"),
          old_vol_name, loaded, dev->drive_index);
@@ -406,7 +407,7 @@ bool unload_autochanger(DCR *dcr, int loaded)
          old_vol_name, loaded, dev->drive_index);
       slot = dcr->VolCatInfo.Slot;
       dcr->VolCatInfo.Slot = loaded;
-      changer = edit_device_codes(dcr, changer,
+      edit_device_codes(dcr, &changer,
                    dcr->device->changer_command, "unload");
       dev->close(dcr);
       Dmsg1(dbglvl, "Run program=%s\n", changer);
@@ -450,7 +451,6 @@ static bool unload_other_drive(DCR *dcr, int slot, bool writing)
    DEVRES *device;
    int retries = 0;                /* wait for device retries */
    int loaded;
-   int i;
 
    if (!changer || !changer->device) {
       return false;
@@ -471,7 +471,7 @@ static bool unload_other_drive(DCR *dcr, int slot, bool writing)
     * The above fails to loop through all devices. It is
     * probably a compiler bug.
     */
-   for (i=0; i < changer->device->size(); i++) {
+   for (int i=0; i < changer->device->size(); i++) {
       device = (DEVRES *)changer->device->get(i);
       dev = device->dev;
       if (!dev) {
@@ -590,8 +590,7 @@ bool unload_dev(DCR *dcr, DEVICE *dev)
    Dmsg3(0/*dbglvl*/, "Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n",
         old_vol_name, dev->get_slot(), dev->drive_index);
 
-   changer_cmd = edit_device_codes(dcr, changer_cmd,
-                dcr->device->changer_command, "unload");
+   edit_device_codes(dcr, &changer_cmd, dcr->device->changer_command, "unload");
    dev->close(dcr);
    Dmsg2(dbglvl, "close dev=%s reserve=%d\n", dev->print_name(),
       dev->num_reserved());
@@ -671,8 +670,7 @@ bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
    changer = get_pool_memory(PM_FNAME);
    lock_changer(dcr);
    /* Now issue the command */
-   changer = edit_device_codes(dcr, changer,
-                 dcr->device->changer_command, cmd);
+   edit_device_codes(dcr, &changer, dcr->device->changer_command, cmd);
    dir->fsend(_("3306 Issuing autochanger \"%s\" command.\n"), cmd);
    bpipe = open_bpipe(changer, timeout, "r");
    if (!bpipe) {
@@ -720,12 +718,14 @@ bail_out:
  *  %c = changer device name
  *  %d = changer drive index
  *  %f = Client's name
+ *  %i = JobId
  *  %j = Job name
  *  %l = archive control channel name
  *  %o = command
  *  %s = Slot base 0
  *  %S = Slot base 1
  *  %v = Volume name
+ *  %V = Volume name from dcr->VolCatInfo first
  *
  *
  *  omsg = edited output message
@@ -733,13 +733,16 @@ bail_out:
  *  cmd = command string (load, unload, ...)
  *
  */
-char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
+void edit_device_codes(DCR *dcr, POOLMEM **omsg, const char *imsg, const char *cmd)
 {
    const char *p;
    const char *str;
    char add[20];
 
-   *omsg = 0;
+   **omsg = '\0';
+   if (imsg == NULL) {
+      return;
+   }
    Dmsg1(1800, "edit_device_codes: %s\n", imsg);
    for (p=imsg; *p; p++) {
       if (*p == '%') {
@@ -757,7 +760,7 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
             str = NPRT(dcr->device->control_name);
             break;
          case 'd':
-            sprintf(add, "%d", dcr->dev->drive_index);
+            sprintf(add, "%u", dcr->dev->drive_index);
             str = add;
             break;
          case 'o':
@@ -771,8 +774,12 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
             sprintf(add, "%d", dcr->VolCatInfo.Slot);
             str = add;
             break;
+	 case 'i':
+	    sprintf(add, "%d", dcr->jcr ? dcr->jcr->JobId : 0);
+	    str = add;
+	    break;
          case 'j':                    /* Job name */
-            str = dcr->jcr->Job;
+            str = dcr->jcr ? dcr->jcr->Job : "";
             break;
          case 'v':
             if (dcr->dev->LoadedVolName[0]) {
@@ -787,6 +794,19 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
                str = dcr->dev->VolHdr.VolumeName;
             }
             break;
+         case 'V':
+            if (dcr->VolCatInfo.VolCatName[0]) {
+               str = dcr->VolCatInfo.VolCatName;
+            } else if (dcr->VolumeName[0]) {
+               str = dcr->VolumeName;
+            } else if (dcr->dev->LoadedVolName[0]) {
+                  str = dcr->dev->LoadedVolName;
+            } else if (dcr->dev->vol && dcr->dev->vol->vol_name) {
+               str = dcr->dev->vol->vol_name;
+            } else {
+               str = dcr->dev->VolHdr.VolumeName;
+            }
+            break;
          case 'f':
             str = NPRT(dcr->jcr->client_name);
             break;
@@ -804,9 +824,8 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
          str = add;
       }
       Dmsg1(1900, "add_str %s\n", str);
-      pm_strcat(&omsg, (char *)str);
-      Dmsg1(1800, "omsg=%s\n", omsg);
+      pm_strcat(omsg, (char *)str);
+      Dmsg1(1800, "omsg=%s\n", *omsg);
    }
-   Dmsg1(800, "omsg=%s\n", omsg);
-   return omsg;
+   Dmsg1(800, "omsg=%s\n", *omsg);
 }
