@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -113,6 +113,7 @@ public:
    /* variables */
    char                *m_cache_fname;
    char                *m_volume_name;
+   char                *m_device_name;
    uint32_t             m_part;
    cloud_driver        *m_driver;
    uint32_t             m_job_id;
@@ -138,6 +139,9 @@ public:
 
    /* truncate cache once transfer is completed (upload)*/
    bool                 m_do_cache_truncate;
+
+   /* for Archive/Glacier restoration */
+   POOLMEM             *m_restore_bucket;
 /* methods :*/
    /* constructor
    * size         : the size in bytes of the transfer
@@ -152,6 +156,7 @@ public:
             transfer_engine *funct,
             const char   *cache_fname,
             const char   *volume_name,
+            const char   *device_name,
             uint32_t     part,
             cloud_driver *driver,
             uint32_t     JobId,
@@ -188,6 +193,8 @@ public:
 
    void set_do_cache_truncate(bool do_cache_truncate);
 
+   void set_restore_bucket(const char *restore_bucket);
+
    /* reset processed size */
    void reset_processed_size();
 
@@ -290,6 +297,7 @@ public:
             transfer_engine *funct,
             POOLMEM      *cache_fname,
             const char   *volume_name,
+            const char   *device_name,
             uint32_t     part,
             cloud_driver *driver,
             uint32_t     JobId,
