@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -215,7 +215,8 @@ struct VOLUME_CAT_INFO {
    uint32_t VolCatRecycles;           /* Number of recycles this volume */
    uint32_t EndFile;                  /* Last file number */
    uint32_t EndBlock;                 /* Last block number */
-
+   uint32_t VolRetention;             /* Volume Retention */
+   
    int32_t  LabelType;                /* Bacula/ANSI/IBM */
    int32_t  Slot;                     /* >0=Slot loaded, 0=nothing, -1=unknown */
    uint32_t VolCatMaxJobs;            /* Maximum Jobs to write to volume */
@@ -233,6 +234,9 @@ struct VOLUME_CAT_INFO {
    utime_t  VolFirstWritten;          /* Time of first write */
    utime_t  VolLastWritten;           /* Time of last write */
    bool     InChanger;                /* Set if vol in current magazine */
+   bool     Protected;                /* Set if the vol is Readonly, worm or immutable */
+   bool     UseProtect;               /* Set if the device can set the volume Readonly, worm or immutable */
+   bool     VolEncrypted;             /* Set if the volume is encrypted */
    bool     is_valid;                 /* set if this data is valid */
    bool     VolEnabled;               /* set if volume enabled */
    bool     VolRecycle;               /* set if volume can be recycled */
@@ -299,6 +303,7 @@ public:
    bool m_shstore_user_lock;          /* set if user set shared lock */
    bool m_shstore_register;           /* set if register key set */
    bool m_shstore_blocked;            /* Set if I am blocked */
+   bool m_delay_adata;                /* Set if we need to ask to read adata */
    bool adata;                        /* set if adata device */
    int label_type;                    /* Bacula/ANSI/IBM label types */
    uint32_t drive_index;              /* Autochanger drive index (base 0) */
@@ -330,6 +335,7 @@ public:
    uint64_t adata_addr;               /* Next adata write address */
 
    uint64_t max_part_size;            /* max part size */
+   uint32_t max_vol_parts_num;        /* Max number of parts in a cloud volume */
    uint64_t part_size;                /* current part size */
    uint32_t part;                     /* current part number (starts at 0) */
    /* state ST_FREESPACE_OK is set if free_space is valid */
@@ -339,7 +345,7 @@ public:
    uint64_t min_free_space;           /* Minimum free disk space */
    int free_space_errno;              /* indicates errno getting freespace */
    bool truncating;                   /* if set, we are currently truncating */
-
+   
    utime_t  vol_poll_interval;        /* interval between polling Vol mount */
    DEVRES *device;                    /* pointer to Device Resource */
    VOLRES *vol;                       /* Pointer to Volume reservation item */
@@ -378,6 +384,8 @@ public:
    devstatmetrics_t devstatmetrics;    /* these are a device metrics for every device */
    bstatcollect *devstatcollector;        /* a pointer to daemon's statcollector */
 
+   BLOCK_CIPHER_CONTEXT *crypto_device_ctx;
+
    /* Methods */
    btime_t get_timer_count(); /* return the last timer interval (ms) */
 
@@ -388,6 +396,7 @@ public:
    void set_worm(bool is_worm) { m_is_worm = is_worm; }
    bool do_checksum() const { return (capabilities & CAP_BLOCKCHECKSUM) != 0; }
    int is_autochanger() const { return capabilities & CAP_AUTOCHANGER; }
+   bool is_virtual_autochanger() const;
    int requires_mount() const { return capabilities & CAP_REQMOUNT; }
    int is_removable() const { return capabilities & CAP_REM; }
    bool is_tape() const { return (dev_type == B_TAPE_DEV ||
@@ -457,12 +466,14 @@ public:
    };
    void get_freespace(uint64_t *freeval, uint64_t *totalval); /* in dev.c */
    void set_freespace(uint64_t freeval, uint64_t totalval, int errnoval, bool valid); /* in dev.c */
-   bool is_fs_nearly_full(uint64_t threshold);
+   virtual bool is_fs_nearly_full(uint64_t threshold);
    bool is_volume_to_unload() const { \
       return m_unload && strcmp(VolHdr.VolumeName, LoadedVolName) == 0; };
    void set_load() { m_load = true; };
    void set_wait() { m_wait = true; };
    void clear_wait() { m_wait = false; };
+   void set_delay_adata() { m_delay_adata = true;};
+   void clear_delay_adata() { m_delay_adata = false;};
    bool must_wait() const { return m_wait; };
    void inc_reserved() { m_num_reserved++; }
    void set_mounted(int val) { if (val) state |= ST_MOUNTED; \
@@ -509,7 +520,7 @@ public:
    void set_slot(int32_t slot);  /* in dev.c */
    void clear_slot();            /* in dev.c */
    bool update_freespace();              /* in dev.c */
-   bool get_os_device_freespace();       /* in dev.c */
+   virtual bool get_os_device_freespace();                      /* in dev.c */
    void notify_newvol_in_attached_dcrs(const char *VolumeName); /* in dev.c */
    void notify_newfile_in_attached_dcrs();/* in dev.c */
    void attach_dcr_to_dev(DCR *dcr);      /* in acquire.c */
@@ -526,6 +537,8 @@ public:
 
 
    /* Virtual functions that can be overridden */
+   virtual void set_file_size(uint64_t val) { file_size = val;};
+   virtual uint64_t update_file_size(uint64_t add) { file_size += add; return file_size; };
    virtual void setVolCatName(const char *name);
    virtual void setVolCatStatus(const char *status);
    virtual void free_dcr_blocks(DCR *dcr);        /* in block_util.c */
@@ -563,11 +576,11 @@ public:
    virtual bool rewind(DCR *dcr);
    virtual bool truncate(DCR *dcr);
    virtual int  truncate_cache(DCR *dcr, const char *VolName, int64_t *size, POOLMEM *&msg) { return 0; };
-   virtual bool get_cloud_volumes_list(DCR* dcr, alist *volumes, POOLMEM *&err) { pm_strcpy(err, "Not implemented"); return false;};
-   virtual bool get_cloud_volume_parts_list(DCR *dcr, const char *VolumeName, ilist *parts, POOLMEM *&err) { pm_strcpy(err, "Not implemented"); return false; };
-   virtual uint32_t get_cloud_upload_transfer_status(POOL_MEM &msg, bool verbose) { pm_strcpy(msg, "Not implemented"); return 0; };
+   virtual bool get_cloud_volumes_list(DCR* dcr, alist *volumes, POOLMEM *&err) { pm_strcpy(err, _("Not implemented")); return false;};
+   virtual bool get_cloud_volume_parts_list(DCR *dcr, const char *VolumeName, ilist *parts, POOLMEM *&err) { pm_strcpy(err, _("Not implemented")); return false; };
+   virtual uint32_t get_cloud_upload_transfer_status(POOL_MEM &msg, bool verbose) { pm_strcpy(msg, _("Not implemented")); return 0; };
    virtual void get_api_cloud_upload_transfer_status(OutputWriter &, bool) {};
-   virtual uint32_t get_cloud_download_transfer_status(POOL_MEM &msg, bool verbose) { pm_strcpy(msg, "Not implemented"); return 0; };
+   virtual uint32_t get_cloud_download_transfer_status(POOL_MEM &msg, bool verbose) { pm_strcpy(msg, _("Not implemented")); return 0; };
    virtual void get_api_cloud_download_transfer_status(OutputWriter &, bool) {};
    virtual bool upload_cache(DCR *dcr, const char *VolName, uint32_t truncate, POOLMEM *&err) {return true; };
    virtual bool open_device(DCR *dcr, int omode) = 0;
@@ -596,6 +609,20 @@ public:
    virtual bool end_of_job(DCR *dcr, uint32_t truncate) {return true; };
    virtual bool is_indexed() { return true; };
    virtual void set_ateof();                    /* in dev.c */
+   /* Methods below are responsible for managing
+    * the append and immutable flags on device-specific volumes */
+   virtual bool set_append_only(const char *vol_name, POOLMEM **error) { pm_strcpy(error, _("Not Implemented")); return false; };
+   virtual bool clear_append_only(const char *vol_name, POOLMEM **error) { pm_strcpy(error, _("Not Implemented")); return false; };
+   virtual bool set_immutable(const char *vol_name, POOLMEM **error) { pm_strcpy(error, _("Not Implemented")); return false; };
+   virtual bool clear_immutable(const char *vol_name, POOLMEM **error) { pm_strcpy(error, _("Not Implemented")); return false; };
+   virtual bool check_volume_protection_time(const char *vol_name) { return true; };
+   virtual bool check_for_immutable(const char *vol_name) { return false; };
+   virtual bool check_for_read_only(int fd, const char *vol_name) { return false; };
+   virtual int set_writable(int fd, const char *vol_name, POOLMEM **error) { pm_strcpy(error, "Not implemented"); errno=ENOSYS; return -1;};
+   virtual int set_readonly(int fd, const char *vol_name, POOLMEM **error) { pm_strcpy(error, "Not implemented"); errno=ENOSYS; return -1;};
+   virtual int set_atime(int fd, const char *vol_name, btime_t val, POOLMEM **error) { pm_strcpy(error, "Not implemented"); errno=ENOSYS; return -1;};
+   virtual int use_protect() { return 0; };
+   virtual int use_volume_encryption();
    virtual const char *print_type() = 0;        /* in dev.c */
    virtual const char *print_driver_type() { return "";};
    virtual const char *print_full_type() { return print_type();};
@@ -697,6 +724,7 @@ public:
    void open_tape_device(DCR *dcr, int omode);   /* in dev.c */
    void open_file_device(DCR *dcr, int omode);   /* in dev.c */
 
+   virtual int rehydrate_record(DCR *dcr, DEV_RECORD *rec) { return 1;};
    virtual bool setup_dedup_rehydration_interface(DCR *dcr) { return false; };
    virtual void free_dedup_rehydration_interface(DCR *dcr) { };
    virtual GetMsg *get_msg_queue(JCR *jcr, BSOCK *sock, int32_t bufsize)
@@ -709,6 +737,11 @@ private:
    bool mount_tape(int mount, int dotimeout); /* in dev.c */
 protected:
    void set_mode(int omode);                     /* in dev.c */
+   bool load_encryption_key(DCR *dcr, const char *operation,
+         const char *volume_name, /* in dev.c */
+         uint32_t *enc_cipher_key_size, unsigned char *enc_cipher_key,
+         uint32_t *master_keyid_size, unsigned char *master_keyid);
+
 };
 inline const char *DEVICE::strerror() const { return errmsg; }
 inline const char *DEVICE::archive_name() const { return dev_name; }
@@ -800,6 +833,7 @@ public:
    int Copy;                          /* identical copy number */
    int Stripe;                        /* RAIT stripe */
    VOLUME_CAT_INFO VolCatInfo;        /* Catalog info for desired volume */
+   /* *** BEEF *** */
    uint32_t crc32(unsigned char *buf, int len, uint32_t expected_crc);
 
    /* *** SIR *** */
@@ -867,6 +901,7 @@ public:
    void mark_volume_in_error();
    void mark_volume_read_only();
    void mark_volume_not_inchanger();
+   void volume_is_unavailable();
    int try_autolabel(bool opened);
    bool find_a_volume();
    bool is_suitable_volume_mounted();
