@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -36,7 +36,7 @@ void dedup_get_limits(int64_t *nofile, int64_t *memlock)
 }
 
 /* dump the status of all dedupengines */
-void list_dedupengines(char *cmd, STATUS_PKT *sp)
+void list_dedupengines(char *cmd, STATUS_PKT *sp, const char *target)
 {
 }
 
