@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -31,7 +31,7 @@
 
 /* Imported subroutines */
 static const int read_dbglvl = 200|DT_VOLUME;
-static const int dbgep = 200|DT_VOLUME;         /* debug execution path */
+static const int dbgep = 400|DT_VOLUME;         /* debug execution path */
 
 /*
  * Read the header record
@@ -63,8 +63,10 @@ static bool read_header(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
          block->adata, block->BlockNumber, block->BlockVer, block->block_len);
    if (block->BlockVer == 1) {
       rhl = RECHDR1_LENGTH;
-   } else {
+   } if (block->BlockVer == 2) {
       rhl = RECHDR2_LENGTH;
+   } else {
+      rhl = RECHDR3_LENGTH;
    }
    if (rec->remlen >= rhl) {
       Dmsg0(dbgep, "=== rpath 2 begin unserial header\n");
@@ -86,7 +88,6 @@ static bool read_header(DCR *dcr, DEV_BLOCK *block, DEV_RECORD *rec)
       if (dcr->dev->have_adata_header(dcr, rec, FileIndex, Stream, VolSessionId)) {
          return true;
       }
-
       block->bufp += rhl;
       block->binbuf -= rhl;
       rec->remlen -= rhl;
@@ -238,16 +239,24 @@ bool read_record_from_block(DCR *dcr,  DEV_RECORD *rec)
 
    Dmsg0(dbgep, "=== rpath 1 Enter read_record_from block\n");
 
+   /* We return from a previous call with aligned volume, we did not read
+    * the data on disk, let's read the next ameta record.
+    */
+   if (rec->rstate == st_header_only) {
+      rec->remainder = 0;
+      rec->rstate = st_header;
+   }
    /* Update the Record number only if we have a new record */
    if (rec->remainder == 0) {
       rec->RecNum = dcr->block->RecNum;
       rec->VolumeName = dcr->CurrentVol->VolumeName; /* From JCR::VolList, freed at the end */
       rec->Addr = rec->StartAddr = dcr->block->BlockAddr;
    }
-
+   rec->BlockVer = dcr->block->BlockVer; /* needed for unser_volume_label() */
+   rec->blkh_options = dcr->block->blkh_options; /* needed for unser_volume_label() */
    /* We read the next record */
    dcr->block->RecNum++;
-
+   
    for ( ;; ) {
       switch (rec->rstate) {
       case st_none:
@@ -264,6 +273,13 @@ bool read_record_from_block(DCR *dcr,  DEV_RECORD *rec)
          }
          continue;
 
+      case st_header_only:
+         /* We come from the st_cont_adata_rechdr state and we may read
+          * or not the data associated
+          */
+         Dmsg0(dbgep, "=== rpath 37 st_header_only\n");
+         goto get_out;
+
       case st_data:
          Dmsg0(dbgep, "=== rpath 37 st_data\n");
          read_data(dcr->block, rec);
