@@ -1,7 +1,7 @@
 /*
    Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2022 Kern Sibbald
+   Copyright (C) 2000-2023 Kern Sibbald
 
    The original author of Bacula is Kern Sibbald, with contributions
    from many others, a complete list can be found in the file AUTHORS.
@@ -115,16 +115,16 @@ static void JmsgVssApiStatus(JCR *jcr, int msg_status, HRESULT hr, const char *a
    case E_ACCESSDENIED:
       errmsg = "The caller does not have sufficient backup privileges or is not an administrator.";
       break;
-   case VSS_E_INVALID_XML_DOCUMENT:
+   case ((HRESULT)VSS_E_INVALID_XML_DOCUMENT):
       errmsg = "The XML document is not valid.";
       break;
-   case VSS_E_OBJECT_NOT_FOUND:
+   case ((HRESULT)VSS_E_OBJECT_NOT_FOUND):
       errmsg = "The specified file does not exist.";
       break;
-   case VSS_E_BAD_STATE:
+   case ((HRESULT)VSS_E_BAD_STATE):
       errmsg = "Object is not initialized; called during restore or not called in correct sequence.";
       break;
-   case VSS_E_WRITER_INFRASTRUCTURE:
+   case ((HRESULT)VSS_E_WRITER_INFRASTRUCTURE):
       errmsg = "The writer infrastructure is not operating properly. Check that the Event Service and VSS have been started, and check for errors associated with those services in the error log.";
       break;
    case VSS_S_ASYNC_CANCELLED:
@@ -277,7 +277,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    IVssBackupComponents* pVssObj = (IVssBackupComponents*)m_pVssObject;
 
    if (!(p_CreateVssBackupComponents && p_VssFreeSnapshotProperties)) {
-      Dmsg2(0, "VSSClientGeneric::Initialize: p_CreateVssBackupComponents=0x%08X, p_VssFreeSnapshotProperties=0x%08X\n", p_CreateVssBackupComponents, p_VssFreeSnapshotProperties);
+      Dmsg2(10, "VSSClientGeneric::Initialize: p_CreateVssBackupComponents=0x%08X, p_VssFreeSnapshotProperties=0x%08X\n", p_CreateVssBackupComponents, p_VssFreeSnapshotProperties);
       Jmsg(m_jcr, M_FATAL, 0, "Entry point CreateVssBackupComponents or VssFreeSnapshotProperties missing.\n");
       return false;
    }
@@ -287,7 +287,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    }
 
    m_VolumeList = New(MTab());  // TODO: See if we do this part only in backup
-   if (!m_VolumeList->get()) {
+   if (!m_VolumeList->load_volumes()) {
       Jmsg(m_jcr, M_ERROR, 0, "Unable to list devices and volumes.\n");
       return false;
    }
@@ -297,7 +297,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    if (!m_bCoInitializeCalled) {
       hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
       if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: CoInitializeEx returned 0x%08X\n", hr);
+         Dmsg1(10, "VSSClientGeneric::Initialize: CoInitializeEx returned 0x%08X\n", hr);
          JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CoInitializeEx");
          errno = b_errno_win32;
          return false;
@@ -315,7 +315,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
    hr = p_CreateVssBackupComponents((IVssBackupComponents**)&m_pVssObject);
    if (FAILED(hr)) {
       berrno be;
-      Dmsg2(0, "VSSClientGeneric::Initialize: CreateVssBackupComponents returned 0x%08X. ERR=%s\n",
+      Dmsg2(10, "VSSClientGeneric::Initialize: CreateVssBackupComponents returned 0x%08X. ERR=%s\n",
             hr, be.bstrerror(b_errno_win32));
       JmsgVssApiStatus(m_jcr, M_FATAL, hr, "CreateVssBackupComponents");
       errno = b_errno_win32;
@@ -331,7 +331,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
       if (dwContext != VSS_CTX_BACKUP) {
          hr = pVssObj->SetContext(dwContext);
          if (FAILED(hr)) {
-            Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->SetContext returned 0x%08X\n", hr);
+            Dmsg1(10, "VSSClientGeneric::Initialize: IVssBackupComponents->SetContext returned 0x%08X\n", hr);
             JmsgVssApiStatus(m_jcr, M_FATAL, hr, "SetContext");
             errno = b_errno_win32;
             return false;
@@ -342,7 +342,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
       // 1. InitializeForBackup
       hr = pVssObj->InitializeForBackup();
       if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->InitializeForBackup returned 0x%08X\n", hr);
+         Dmsg1(10, "VSSClientGeneric::Initialize: IVssBackupComponents->InitializeForBackup returned 0x%08X\n", hr);
          JmsgVssApiStatus(m_jcr, M_FATAL, hr, "InitializeForBackup");
          errno = b_errno_win32; 
          return false;
@@ -360,13 +360,13 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
          backup_type = VSS_BT_INCREMENTAL;
          break;
       default:
-         Dmsg1(0, "VSSClientGeneric::Initialize: unknown backup level %d\n", m_jcr->getJobLevel());
+         Dmsg1(10, "VSSClientGeneric::Initialize: unknown backup level %d\n", m_jcr->getJobLevel());
          backup_type = VSS_BT_FULL;
          break;
       }
       hr = pVssObj->SetBackupState(true, true, backup_type, false); /* FIXME: need to support partial files - make last parameter true when done */
       if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->SetBackupState returned 0x%08X\n", hr);
+         Dmsg1(10, "VSSClientGeneric::Initialize: IVssBackupComponents->SetBackupState returned 0x%08X\n", hr);
          JmsgVssApiStatus(m_jcr, M_FATAL, hr, "SetBackupState");
          errno = b_errno_win32;
          return false;
@@ -375,7 +375,7 @@ bool VSSClientGeneric::Initialize(DWORD dwContext, bool bDuringRestore)
       // 3. GatherWriterMetaData
       hr = pVssObj->GatherWriterMetadata(&pAsync1.p);
       if (FAILED(hr)) {
-         Dmsg1(0, "VSSClientGeneric::Initialize: IVssBackupComponents->GatherWriterMetadata returned 0x%08X\n", hr);
+         Dmsg1(10, "VSSClientGeneric::Initialize: IVssBackupComponents->GatherWriterMetadata returned 0x%08X\n", hr);
          JmsgVssApiStatus(m_jcr, M_FATAL, hr, "GatherWriterMetadata");
          errno = b_errno_win32;
          return false;
@@ -490,7 +490,8 @@ bool VSSClientGeneric::CreateSnapshots(alist *mount_points)
          MTabEntry *elt = (MTabEntry*)m_VolumeList->entries->search(p, volume_cmp);
          ASSERT2(elt, "Should find the volume in the list");
          Jmsg(m_jcr, M_INFO, 0, "    Snapshot mount point: %ls\n", elt->first());
-         Dmsg1(50, "AddToSnapshot OK for Vol: %ls\n", p);
+         Dmsg2(50, "AddToSnapshot OK for %ls vol=%ls\n", elt->first(), p);
+         /* Here the snapshot don't exist yet, elt->shadowCopyName == NULL */
       } else {
          //Dmsg1(50, "AddToSnapshot() failed for Vol: %ls\n", (LPWSTR)volume.c_str());
          Dmsg2(50, "AddToSnapshot() failed for path: %ls hresult=0x%x\n", p, hresult);
